#include "precompile.h"

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/vec2.hpp>

#include "buff/select_target_with_self_pos.h"

#include "creature.h"

#include "mt/Buff.h"

void SelectTargetWithSelfPosBuff::Activate()
{
    hold_param3_ = meta->GetBuffParam3(this);
    hold_param4_ = meta->GetBuffParam4(this);
    std::vector<Creature*> targets;
    bool include_car = false;
    bool exclude_passenger = false;
    if (!meta->_buff_param8_int_set.empty()) {
        include_car = meta->_buff_param8_int_set.find(1) != meta->_buff_param8_int_set.end();
        exclude_passenger = meta->_buff_param8_int_set.find(2) != meta->_buff_param8_int_set.end();
    }
    owner->TraverseCreatures
        (
         [this, &targets, include_car, exclude_passenger] (Creature* c, bool& stop)
         {
             if (c->dead ||
                 (c->IsCar() && !include_car) ||
                 a8::HasBitFlag(c->status, CS_Disable) ||
                 c->HasBuffEffect(kBET_Dive)
                 ) {
                 return;
             }
             if (exclude_passenger) {
                 if (c->HasBuffEffect(kBET_Driver) &&
                     c->HasBuffEffect(kBET_Passenger)) {
                     return;
                 }
             }
             float distance = owner->GetPos().Distance2D2(c->GetPos());
             float angle = 0.0f;
             if (distance > 0.0001f) {
                 glm::vec3 dir = owner->GetAttackDir();
                 if (GlmHelper::IsZero(dir)){
                         dir = c->GetPos().ToGlmVec3() - owner->GetPos().ToGlmVec3();
                         GlmHelper::Normalize(dir);
                 }
                 angle = GlmHelper::CalcAngle(dir,
                                              c->GetPos().ToGlmVec3() - owner->GetPos().ToGlmVec3());
             }
             if (distance < hold_param3_ && std::abs(angle) <= glm::radians(hold_param4_) / 2.0f / A8_PI) {
                 switch (meta->_int_buff_param1) {
                 case kBST_All:
                     {
                         targets.push_back(c);
                     }
                     break;
                 case kBST_Self:
                     {
                         if (c == owner) {
                             targets.push_back(c);
                         }
                     }
                     break;
                 case kBST_FriendlyIncludeSelf:
                     {
                         if (c->team_id == owner->team_id) {
                             targets.push_back(c);
                         }
                     }
                     break;
                 case kBST_FriendlyExcludeSelf:
                     {
                         if (c->team_id == owner->team_id && c != owner) {
                             targets.push_back(c);
                         }
                     }
                     break;
                 case kBST_Enemy:
                     {
                         if (c->team_id != owner->team_id) {
                             targets.push_back(c);
                         }
                     }
                     break;
                 case kBST_EnemyAndSelf:
                     {
                         if (c->team_id != owner->team_id || c == owner) {
                             targets.push_back(c);
                         }
                     }
                     break;
                 default:
                     {
                     }
                     break;
                 }
             }
         }
         );
    for (auto& target : targets) {
        for (int buff_id : meta->_buff_param2_int_list) {
            target->TryAddBuff(caster_.Get(), buff_id, skill_meta);
        }
    }
}

void SelectTargetWithSelfPosBuff::Deactivate()
{

}
