#include "precompile.h"

#include "ingamevoice.h"
#include "room.h"
#include "human.h"
#include "team.h"
#include "stats.h"

#include "mt/InGameVoice.h"

static void IncIntMap(std::map<int, int>& int_map, int key)
{
    auto itr = int_map.find(key);
    if (itr != int_map.end()) {
        ++itr->second;
    } else {
        int_map[key] = 1;
    }
}

static int GetIntMap(std::map<int, int>& int_map, int key)
{
    auto itr = int_map.find(key);
    if (itr != int_map.end()) {
        return itr->second;
    } else {
        return 0;
    }
}

static int GetInt64Map(std::map<int, long long>& int_map, int key)
{
    auto itr = int_map.find(key);
    if (itr != int_map.end()) {
        return itr->second;
    } else {
        return 0;
    }
}

InGameVoice::InGameVoice(Room* room)
{
    room_ = room;
}

InGameVoice::~InGameVoice()
{

}

void InGameVoice::Init()
{

}

void InGameVoice::UnInit()
{

}

void InGameVoice::OnHumanBeKill(int killer_id, Human* deader)
{
    personal_nodead_series_kills_.erase(deader->GetUniId());
    if (killer_id == deader->GetUniId()) {
        return;
    }
    Human* killer = room_->GetHumanByUniId(killer_id);
    if (!killer) {
        return;
    }
    ++global_kills_;
    if (global_kills_ > 0) {
        if (room_->GetFrameNo() - GetLastKillFrameNo(killer_id) >
            SERVER_FRAME_RATE * mt::InGameVoice::s_series_kill_interval) {
            personal_series_kills_.erase(killer_id);
        }
        IncIntMap(personal_kills_, killer_id);
        IncIntMap(personal_series_kills_, killer_id);
        if (!killer->dead) {
            IncIntMap(personal_nodead_series_kills_, killer_id);
        }
    }
    personal_last_kill_frameno_[killer_id] = room_->GetFrameNo();
    TriggerEvent(killer, deader);
}

void InGameVoice::Notify(Human* killer, Human* deader, const mt::InGameVoice* meta,
                         std::function<bool (Human*)>& filter_cb)
{
    {
        switch (meta->play_scene()) {
        case 1:
            {
                if (!room_->IsMobaModeRoom()) {
                    return;
                }
            }
            break;
        case 2:
            {
                if (room_->IsMobaModeRoom()) {
                    return;
                }
            }
            break;
        case 3:
            {
            }
            break;
        default:
            {
            }
            break;
        }
    }
    auto p = std::make_shared<FightVoice>();
    {
        p->voice_id = meta->sound_id();
        switch (meta->notify_fields()) {
        case 1:
            {
                p->killer_id = killer->GetUniId();
            }
            break;
        case 2:
            {
                p->dead_id = deader->GetUniId();
            }
            break;
        case 3:
            {
                p->killer_id = killer->GetUniId();
                p->dead_id = deader->GetUniId();
            }
            break;
        default:
            {
                return;
            }
            break;
        }
    }
    switch (meta->GetNotifyObject()) {
    case InGameVoiceNotifyObject_e::kGlobal:
        {
            room_->TraverseHumanList
                (
                 [p, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         hum->GetFrameData().voices_.push_back(p);
                     }
                     return true;
                 });
        }
        break;

    case InGameVoiceNotifyObject_e::kKiller:
        {
            if (filter_cb(killer)) {
                killer->GetFrameData().voices_.push_back(p);
            }
        }
        break;
    case InGameVoiceNotifyObject_e::kNoKiller:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum != killer) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;
    case InGameVoiceNotifyObject_e::kKillerTeam:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum->GetTeam() == killer->GetTeam()) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;
    case InGameVoiceNotifyObject_e::kNoKillerTeam:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum->GetTeam() != killer->GetTeam()) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;

    case InGameVoiceNotifyObject_e::kDead:
        {
            if (filter_cb(deader)) {
                deader->GetFrameData().voices_.push_back(p);
            }
        }
        break;
    case InGameVoiceNotifyObject_e::kNoDead:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum != deader) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;
    case InGameVoiceNotifyObject_e::kDeadTeam:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum->GetTeam() == deader->GetTeam()) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;
    case InGameVoiceNotifyObject_e::kNoDeadTeam:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum->GetTeam() != deader->GetTeam()) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;
    case InGameVoiceNotifyObject_e::kTeammateExcludeDead:
        {
            room_->TraverseHumanList
                (
                 [p, killer, deader, &filter_cb] (Human* hum) -> bool
                 {
                     if (filter_cb(hum)) {
                         if (hum->GetTeam() == deader->GetTeam() &&
                             hum != deader) {
                             hum->GetFrameData().voices_.push_back(p);
                         }
                     }
                     return true;
                 });
        }
        break;

    default:
        {
        }
        break;
    }
}

void InGameVoice::TriggerEvent(Human* killer, Human* deader)
{
    bool is_first_boold = global_kills_ == 1;
    bool sent = false;
    std::function<bool (Human*)> filter_cb =
        [] (Human*) -> bool
        {
            return true;
        };
    {
        int val = global_kills_;
        mt::InGameVoice::Traverse
            (
             room_->IsMobaModeRoom(),
             InGameVoiceEventType_e::kGlobalKills,
             [this, killer, deader, val, &sent, &filter_cb] (const mt::InGameVoice* meta) -> bool
             {
                 if (meta->MatchCond(val)) {
                     Notify(killer, deader, meta, filter_cb);
                     sent = true;
                 }
                 return true;
             });
    }
    if (sent) {
        return;
    }
    if (is_first_boold) {
        return;
    }
    std::map<int, long long> killer_flags;
    filter_cb =
        [&killer_flags] (Human* hum) -> bool
        {
            if (killer_flags.find(hum->GetUniId()) != killer_flags.end()) {
                return false;
            }
            return true;
        };
    {
        bool sent1 = false;
        if (!IsSpecSeriesKill(killer, deader)) {
            int val = GetIntMap(personal_series_kills_, killer->GetUniId());
            mt::InGameVoice::Traverse
                (
                 room_->IsMobaModeRoom(),
                 InGameVoiceEventType_e::kSeriesKills,
                 [this, killer, deader, val, &sent1, &filter_cb, &killer_flags]
                 (const mt::InGameVoice* meta) -> bool
                 {
                     if (meta->MatchCond(val)) {
                         Notify(killer, deader, meta, filter_cb);
                         killer_flags[killer->GetUniId()] = 1;
                         sent1 = true;
                     }
                     return true;
                 });
        }
        if (!sent1) {
            int val = GetIntMap(personal_nodead_series_kills_, killer->GetUniId());
            mt::InGameVoice::Traverse
                (
                 room_->IsMobaModeRoom(),
                 InGameVoiceEventType_e::kNoDeadSeriesKills,
                 [this, killer, deader, val, &sent, &filter_cb, &killer_flags]
                 (const mt::InGameVoice* meta) -> bool
                 {
                     if (meta->MatchCond(val)) {
                         if (killer->IsPlayer()) {
                             int i = 0;
#ifdef MYDEBUG
                             a8::XPrintf("nodead_skills val:%d meta_id:%d killer_flags:%d\n",
                                         {
                                          val,
                                          meta->id(),
                                          killer_flags.size()
                                         });
#endif
                         }
                         Notify(killer, deader, meta, filter_cb);
                         killer_flags[killer->GetUniId()] = 1;
                         sent = true;
                     }
                     return true;
                 });
            if (sent) {
                return;
            }
        }
    }
    {
        {
            int val = GetIntMap(personal_nodead_series_kills_, killer->GetUniId());
            mt::InGameVoice::Traverse
                (
                 room_->IsMobaModeRoom(),
                 InGameVoiceEventType_e::kPersonalExceptFirstBooldKills,
                 [this, killer, deader, val, &filter_cb, &killer_flags]
                 (const mt::InGameVoice* meta) -> bool
                 {
                     if (meta->MatchCond(val)) {
                         Notify(killer, deader, meta, filter_cb);
                         killer_flags[killer->GetUniId()] = 1;
                     }
                     return true;
                 });
        }
    }
    {
        if (deader->GetTeam()->AllIsDead()) {
            if (room_->IsMobaModeRoom()) {
                mt::InGameVoice::Traverse
                    (
                     room_->IsMobaModeRoom(),
                     InGameVoiceEventType_e::kTeamAllDead,
                     [this, killer, deader, &filter_cb, &killer_flags]
                     (const mt::InGameVoice* meta) -> bool
                     {
                         Notify(killer, deader, meta, filter_cb);
                         return true;
                     });
            } else {
                Fail(deader->GetTeam());
                if (room_->GetAliveTeamNum() == 1) {
                    Victory(room_->GetAliveTeam());
                }
            }
        } else {
            mt::InGameVoice::Traverse
                (
                 room_->IsMobaModeRoom(),
                 InGameVoiceEventType_e::kHeroBeKill,
                 [this, killer, deader, &filter_cb, &killer_flags]
                 (const mt::InGameVoice* meta) -> bool
                 {
                     Notify(killer, deader, meta, filter_cb);
                     return true;
                 });
        }
    }

}

void InGameVoice::OnHumanRevive(Human* hum)
{

}

void InGameVoice::Fail(Team* team)
{
    auto voice_meta = mt::InGameVoice::GetById(99);
    if (voice_meta) {
        auto p = std::make_shared<FightVoice>();
        p->voice_id = voice_meta->sound_id();
        team->TraverseMembers
            (
             [p] (Human* hum) -> bool
             {
                 hum->GetFrameData().voices_.push_back(p);
                 return true;
             });
    }
}

void InGameVoice::Victory(Team* team)
{
    auto voice_meta = mt::InGameVoice::GetById(100);
    if (voice_meta) {
        auto p = std::make_shared<FightVoice>();
        p->voice_id = voice_meta->sound_id();
        team->TraverseMembers
            (
             [p] (Human* hum) -> bool
             {
                 hum->GetFrameData().voices_.push_back(p);
                 return true;
             });
    }
}


bool InGameVoice::IsSpecSeriesKill(Human* killer, Human* deader)
{
    bool ok = false;
    int val = GetIntMap(personal_nodead_series_kills_, killer->GetUniId());
    mt::InGameVoice::Traverse
        (
         room_->IsMobaModeRoom(),
         InGameVoiceEventType_e::kNoDeadSeriesKills,
         [this, killer, deader, val, &ok]
         (const mt::InGameVoice* meta) -> bool
         {
             if (meta->MatchCond(val)) {
                 if (meta->GetCondType() == InGameVoiceCondType_e::kGT ||
                     meta->GetCondType() == InGameVoiceCondType_e::kGE) {
                     ok = true;
                     return false;
                 }
             }
             return true;
         });
    return ok;
}

long long InGameVoice::GetLastKillFrameNo(int killer_id)
{
    return GetInt64Map(personal_last_kill_frameno_, killer_id);
}
