#pragma once

#include <a8/singleton.h>

#include <f8/utils.h>

template<typename T>
void SafeCallStaticPreInit(...) { };
template<typename T, void() = &T::StaticPreInit>
void SafeCallStaticPreInit(int) { T::StaticPreInit(); };

template<typename T>
void SafeCallStaticPostInit(...) { };
template<typename T, void() = &T::StaticPostInit>
void SafeCallStaticPostInit(int) { T::StaticPostInit(); };

template<typename T>
void SafeCallInit1(...) { };
template<typename T, void (T::*)() = &T::Init1>
void SafeCallInit1(int)
{
 T::MutTraverse
 (
  [] (T* meta, bool& stop)
  {
      meta->Init1();
  });
};

template<typename T>
void SafeCallInit2(...) { };
template<typename T, void (T::*)() = &T::Init2>
void SafeCallInit2(int)
{
 T::MutTraverse
 (
  [] (T* meta, bool& stop)
  {
      meta->Init2();
  });
};

template<typename T>
void SafeCallInit3(...) { };
template<typename T, void (T::*)() = &T::Init3>
void SafeCallInit3(int)
{
 T::MutTraverse
 (
  [] (T* meta, bool& stop)
  {
      meta->Init3();
  });
};

namespace mt
{

    struct MetaTable
    {
        std::function<void()> destory_cb;
        std::function<void()> static_pre_init_cb;
        std::function<void()> load_cb;
        std::vector<std::function<void()>> init_cbs;
        std::function<void()> static_post_init_cb;
    };

    class MetaMgr : public a8::Singleton<MetaMgr>
    {

    private:
        MetaMgr() {};
        friend class a8::Singleton<MetaMgr>;

    public:
        void Init();
        void UnInit();

        void RegMetaTables();
        void Load();

        template<class T>
        void RegMetaTable(const std::string& dir)
        {
            auto p = std::make_shared<mt::MetaTable>();
            p->static_pre_init_cb =
                [] ()
                {
                 SafeCallStaticPreInit<T>(0);
                };
            p->destory_cb =
                [] ()
                {
                    for (T* item : T::raw_list) {
                        delete item;
                    }
                    T::raw_list.clear();
                    T::id_hash.clear();
                    T::name_hash.clear();
                };
            p->load_cb =
                [dir] ()
                {
                 f8::ReadJsonMetaFile(dir + T::table_name, T::raw_list);
                 T dummy;
                 std::shared_ptr<a8::reflect::Class> cls = dummy.GetClass();
                 a8::reflect::Field* pk_field = cls->GetFieldByName(T::prim_key);
                 int id = 0;
                 for (auto item : T::raw_list) {
                     switch (T::table_type) {
                     case 0:
                         {
                             T::id_hash[id++] = item;
                         }
                         break;
                     case 1:
                         {
                             long long key = cls->GetFieldValueAsInt64
                                 (
                                  pk_field,
                                  (((char*)item)) + pk_field->offset
                                  );
                             T::id_hash[key] = item;
                         }
                         break;
                     case 2:
                         {
                             std::string key = cls->GetFieldValueAsString
                                 (
                                  pk_field,
                                  (((char*)item)) + pk_field->offset
                                  );
                             T::name_hash[key] = item;
                         }
                         break;
                     default:
                         {
                             abort();
                         }
                         break;
                     }
                 }
                };
            p->init_cbs =
                {
                 [] ()
                 {
                  SafeCallInit1<T>(0);
                 },
                 [] ()
                 {
                  SafeCallInit2<T>(0);
                 },
                 [] ()
                 {
                  SafeCallInit3<T>(0);
                 },
                };
            p->static_post_init_cb =
                [] ()
                {
                 SafeCallStaticPostInit<T>(0);
                };
            meta_tables.push_back(p);
        }

        const std::string GetResDir() { return res_path_; }
    private:
        std::vector<std::shared_ptr<MetaTable>> meta_tables;
        std::string res_path_;
    };

}
