#include <a8/a8.h>
#include <a8/reflect.h>

#include "mtb/Parameter.h"
#include "mtb/Attr.h"
#include "mtb/Map.h"
#include "mtb/MapMode.h"
#include "mtb/MobaRoom.h"
#include "mtb/InGameVoice.h"
#include "mtb/MapArea.h"
#include "mtb/MapThing.h"
#include "mtb/SafeArea.h"
#include "mtb/SafeAreaSafePoint.h"
#include "mtb/SafeAreaPos.h"
#include "mtb/Item.h"
#include "mtb/Equip.h"
#include "mtb/Hero.h"
#include "mtb/Robot.h"
#include "mtb/Skill.h"
#include "mtb/SkillNumber.h"
#include "mtb/NpcStandard.h"
#include "mtb/Buff.h"
#include "mtb/Drop.h"
#include "mtb/AirDrop.h"
#include "mtb/AirRaid.h"
#include "mtb/AirLine.h"
#include "mtb/RankReward.h"
#include "mtb/RankPoint.h"
#include "mtb/KillReward.h"
#include "mtb/KillPoint.h"
#include "mtb/AI.h"
#include "mtb/Text.h"
#include "mtb/HeroQuality.h"
#include "mtb/GunQuality.h"
#include "mtb/FormulaPvp.h"
#include "mtb/PveGemini.h"
#include "mtb/PveGeminiContent.h"
#include "mtb/PveGeminiMode.h"
#include "mtb/RankRoom.h"
#include "mtb/WorldObject.h"
#include "mtb/MergeItem.h"
#include "mtb/MapThingGroup.h"
#include "mtb/BattleBasicAttribute.h"
#include "mtb/BattleLevelAttribute.h"
#include "mtb/BattleRandAttribute.h"
#include "mtb/BattleWeaponAttribute.h"
#include "mtb/BattleUnit.h"
#include "mtb/Condition.h"
#include "mtb/Distribution.h"
#include "mtb/LootConfig.h"
#include "mtb/BattleHeroGrow.h"

namespace mtb
{
    std::shared_ptr<a8::reflect::Class> Parameter::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Parameter", 2, 0);
        meta_class->SetSimpleField(0, "param_name", a8::reflect::ET_STRING, my_offsetof2(Parameter, param_name_));
        meta_class->SetSimpleField(1, "param_value", a8::reflect::ET_STRING, my_offsetof2(Parameter, param_value_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Attr::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Attr", 3, 0);
        meta_class->SetSimpleField(0, "attr_id", a8::reflect::ET_INT32, my_offsetof2(Attr, attr_id_));
        meta_class->SetSimpleField(1, "attr_cname", a8::reflect::ET_STRING, my_offsetof2(Attr, attr_cname_));
        meta_class->SetSimpleField(2, "attr_ename", a8::reflect::ET_STRING, my_offsetof2(Attr, attr_ename_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Map::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Map", 30, 0);
        meta_class->SetSimpleField(0, "map_id", a8::reflect::ET_INT32, my_offsetof2(Map, map_id_));
        meta_class->SetSimpleField(1, "template_list", a8::reflect::ET_STRING, my_offsetof2(Map, template_list_));
        meta_class->SetSimpleField(2, "map_name", a8::reflect::ET_STRING, my_offsetof2(Map, map_name_));
        meta_class->SetSimpleField(3, "map_width", a8::reflect::ET_FLOAT, my_offsetof2(Map, map_width_));
        meta_class->SetSimpleField(4, "map_height", a8::reflect::ET_FLOAT, my_offsetof2(Map, map_height_));
        meta_class->SetSimpleField(5, "airdrops", a8::reflect::ET_STRING, my_offsetof2(Map, airdrops_));
        meta_class->SetSimpleField(6, "terminator_airdrop", a8::reflect::ET_INT32, my_offsetof2(Map, terminator_airdrop_));
        meta_class->SetSimpleField(7, "player", a8::reflect::ET_INT32, my_offsetof2(Map, player_));
        meta_class->SetSimpleField(8, "refresh_robot", a8::reflect::ET_STRING, my_offsetof2(Map, refresh_robot_));
        meta_class->SetSimpleField(9, "map_mode", a8::reflect::ET_INT32, my_offsetof2(Map, map_mode_));
        meta_class->SetSimpleField(10, "safearea", a8::reflect::ET_STRING, my_offsetof2(Map, safearea_));
        meta_class->SetSimpleField(11, "game_start_buff_list", a8::reflect::ET_STRING, my_offsetof2(Map, game_start_buff_list_));
        meta_class->SetSimpleField(12, "map_pic", a8::reflect::ET_STRING, my_offsetof2(Map, map_pic_));
        meta_class->SetSimpleField(13, "first_safearea_center", a8::reflect::ET_STRING, my_offsetof2(Map, first_safearea_center_));
        meta_class->SetSimpleField(14, "init_gas_ring", a8::reflect::ET_INT32, my_offsetof2(Map, init_gas_ring_));
        meta_class->SetSimpleField(15, "airraids", a8::reflect::ET_STRING, my_offsetof2(Map, airraids_));
        meta_class->SetSimpleField(16, "car_num_limit", a8::reflect::ET_STRING, my_offsetof2(Map, car_num_limit_));
        meta_class->SetSimpleField(17, "scale", a8::reflect::ET_FLOAT, my_offsetof2(Map, scale_));
        meta_class->SetSimpleField(18, "map_collider", a8::reflect::ET_STRING, my_offsetof2(Map, map_collider_));
        meta_class->SetSimpleField(19, "world_object_file", a8::reflect::ET_STRING, my_offsetof2(Map, world_object_file_));
        meta_class->SetSimpleField(20, "terrain_file", a8::reflect::ET_STRING, my_offsetof2(Map, terrain_file_));
        meta_class->SetSimpleField(21, "map_type", a8::reflect::ET_INT32, my_offsetof2(Map, map_type_));
        meta_class->SetSimpleField(22, "team", a8::reflect::ET_INT32, my_offsetof2(Map, team_));
        meta_class->SetSimpleField(23, "star_condition", a8::reflect::ET_INT32, my_offsetof2(Map, star_condition_));
        meta_class->SetSimpleField(24, "is_open", a8::reflect::ET_INT32, my_offsetof2(Map, is_open_));
        meta_class->SetSimpleField(25, "ground_sampling_pos", a8::reflect::ET_STRING, my_offsetof2(Map, ground_sampling_pos_));
        meta_class->SetSimpleField(26, "navmesh_file", a8::reflect::ET_STRING, my_offsetof2(Map, navmesh_file_));
        meta_class->SetSimpleField(27, "moba_time", a8::reflect::ET_INT32, my_offsetof2(Map, moba_time_));
        meta_class->SetSimpleField(28, "is_moba", a8::reflect::ET_INT32, my_offsetof2(Map, is_moba_));
        meta_class->SetSimpleField(29, "support_sandtable", a8::reflect::ET_INT32, my_offsetof2(Map, support_sandtable_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MapMode::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MapMode", 7, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(MapMode, id_));
        meta_class->SetSimpleField(1, "mode", a8::reflect::ET_INT32, my_offsetof2(MapMode, mode_));
        meta_class->SetSimpleField(2, "mapMode", a8::reflect::ET_INT32, my_offsetof2(MapMode, mapMode_));
        meta_class->SetSimpleField(3, "is_open", a8::reflect::ET_INT32, my_offsetof2(MapMode, is_open_));
        meta_class->SetSimpleField(4, "map_list", a8::reflect::ET_STRING, my_offsetof2(MapMode, map_list_));
        meta_class->SetSimpleField(5, "level_require", a8::reflect::ET_INT32, my_offsetof2(MapMode, level_require_));
        meta_class->SetSimpleField(6, "parameter1", a8::reflect::ET_INT32, my_offsetof2(MapMode, parameter1_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MobaRoom::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MobaRoom", 8, 0);
        meta_class->SetSimpleField(0, "map_id", a8::reflect::ET_INT32, my_offsetof2(MobaRoom, map_id_));
        meta_class->SetSimpleField(1, "revive_time", a8::reflect::ET_INT32, my_offsetof2(MobaRoom, revive_time_));
        meta_class->SetSimpleField(2, "game_time", a8::reflect::ET_INT32, my_offsetof2(MobaRoom, game_time_));
        meta_class->SetSimpleField(3, "tower_interval", a8::reflect::ET_INT32, my_offsetof2(MobaRoom, tower_interval_));
        meta_class->SetSimpleField(4, "tower_recover_hp_rate", a8::reflect::ET_FLOAT, my_offsetof2(MobaRoom, tower_recover_hp_rate_));
        meta_class->SetSimpleField(5, "towers", a8::reflect::ET_STRING, my_offsetof2(MobaRoom, towers_));
        meta_class->SetSimpleField(6, "monster_revive", a8::reflect::ET_STRING, my_offsetof2(MobaRoom, monster_revive_));
        meta_class->SetSimpleField(7, "batterys", a8::reflect::ET_STRING, my_offsetof2(MobaRoom, batterys_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> InGameVoice::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("InGameVoice", 9, 0);
        meta_class->SetSimpleField(0, "event_type", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, event_type_));
        meta_class->SetSimpleField(1, "sound_id", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, sound_id_));
        meta_class->SetSimpleField(2, "cond", a8::reflect::ET_STRING, my_offsetof2(InGameVoice, cond_));
        meta_class->SetSimpleField(3, "notify_object", a8::reflect::ET_STRING, my_offsetof2(InGameVoice, notify_object_));
        meta_class->SetSimpleField(4, "notify_fields", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, notify_fields_));
        meta_class->SetSimpleField(5, "play_interval", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, play_interval_));
        meta_class->SetSimpleField(6, "series_kill_interval", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, series_kill_interval_));
        meta_class->SetSimpleField(7, "play_scene", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, play_scene_));
        meta_class->SetSimpleField(8, "id", a8::reflect::ET_INT32, my_offsetof2(InGameVoice, id_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MapArea::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MapArea", 6, 0);
        meta_class->SetSimpleField(0, "map_id", a8::reflect::ET_INT32, my_offsetof2(MapArea, map_id_));
        meta_class->SetSimpleField(1, "area_type", a8::reflect::ET_INT32, my_offsetof2(MapArea, area_type_));
        meta_class->SetSimpleField(2, "area_subtype", a8::reflect::ET_INT32, my_offsetof2(MapArea, area_subtype_));
        meta_class->SetSimpleField(3, "area_center", a8::reflect::ET_STRING, my_offsetof2(MapArea, area_center_));
        meta_class->SetSimpleField(4, "area_width", a8::reflect::ET_FLOAT, my_offsetof2(MapArea, area_width_));
        meta_class->SetSimpleField(5, "area_height", a8::reflect::ET_FLOAT, my_offsetof2(MapArea, area_height_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MapThing::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MapThing", 37, 0);
        meta_class->SetSimpleField(0, "thing_id", a8::reflect::ET_INT32, my_offsetof2(MapThing, thing_id_));
        meta_class->SetSimpleField(1, "thing_type", a8::reflect::ET_INT32, my_offsetof2(MapThing, thing_type_));
        meta_class->SetSimpleField(2, "time", a8::reflect::ET_INT32, my_offsetof2(MapThing, time_));
        meta_class->SetSimpleField(3, "type", a8::reflect::ET_INT32, my_offsetof2(MapThing, type_));
        meta_class->SetSimpleField(4, "height", a8::reflect::ET_INT32, my_offsetof2(MapThing, height_));
        meta_class->SetSimpleField(5, "width", a8::reflect::ET_INT32, my_offsetof2(MapThing, width_));
        meta_class->SetSimpleField(6, "hp", a8::reflect::ET_INT32, my_offsetof2(MapThing, hp_));
        meta_class->SetSimpleField(7, "damage", a8::reflect::ET_FLOAT, my_offsetof2(MapThing, damage_));
        meta_class->SetSimpleField(8, "damage_dia", a8::reflect::ET_FLOAT, my_offsetof2(MapThing, damage_dia_));
        meta_class->SetSimpleField(9, "drop", a8::reflect::ET_STRING, my_offsetof2(MapThing, drop_));
        meta_class->SetSimpleField(10, "is_door", a8::reflect::ET_INT32, my_offsetof2(MapThing, is_door_));
        meta_class->SetSimpleField(11, "is_house", a8::reflect::ET_INT32, my_offsetof2(MapThing, is_house_));
        meta_class->SetSimpleField(12, "is_tree", a8::reflect::ET_INT32, my_offsetof2(MapThing, is_tree_));
        meta_class->SetSimpleField(13, "house_id", a8::reflect::ET_INT32, my_offsetof2(MapThing, house_id_));
        meta_class->SetSimpleField(14, "buff_list", a8::reflect::ET_STRING, my_offsetof2(MapThing, buff_list_));
        meta_class->SetSimpleField(15, "explosion_effect", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_effect_));
        meta_class->SetSimpleField(16, "explosion_interval", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_interval_));
        meta_class->SetSimpleField(17, "explosion_dmg_delay", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_dmg_delay_));
        meta_class->SetSimpleField(18, "explosion_times", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_times_));
        meta_class->SetSimpleField(19, "explosion_float", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_float_));
        meta_class->SetSimpleField(20, "preexplosion_summon", a8::reflect::ET_STRING, my_offsetof2(MapThing, preexplosion_summon_));
        meta_class->SetSimpleField(21, "monster_list", a8::reflect::ET_STRING, my_offsetof2(MapThing, monster_list_));
        meta_class->SetSimpleField(22, "special_damage_type", a8::reflect::ET_STRING, my_offsetof2(MapThing, special_damage_type_));
        meta_class->SetSimpleField(23, "receive_special_damage_type", a8::reflect::ET_STRING, my_offsetof2(MapThing, receive_special_damage_type_));
        meta_class->SetSimpleField(24, "param1", a8::reflect::ET_STRING, my_offsetof2(MapThing, param1_));
        meta_class->SetSimpleField(25, "param2", a8::reflect::ET_STRING, my_offsetof2(MapThing, param2_));
        meta_class->SetSimpleField(26, "interaction_mode", a8::reflect::ET_INT32, my_offsetof2(MapThing, interaction_mode_));
        meta_class->SetSimpleField(27, "view_mode", a8::reflect::ET_INT32, my_offsetof2(MapThing, view_mode_));
        meta_class->SetSimpleField(28, "bullet_hit", a8::reflect::ET_INT32, my_offsetof2(MapThing, bullet_hit_));
        meta_class->SetSimpleField(29, "collision_hit", a8::reflect::ET_INT32, my_offsetof2(MapThing, collision_hit_));
        meta_class->SetSimpleField(30, "explosion_hit", a8::reflect::ET_INT32, my_offsetof2(MapThing, explosion_hit_));
        meta_class->SetSimpleField(31, "sweep_tags", a8::reflect::ET_STRING, my_offsetof2(MapThing, sweep_tags_));
        meta_class->SetSimpleField(32, "prebattle_hide", a8::reflect::ET_INT32, my_offsetof2(MapThing, prebattle_hide_));
        meta_class->SetSimpleField(33, "life_time", a8::reflect::ET_INT32, my_offsetof2(MapThing, life_time_));
        meta_class->SetSimpleField(34, "summon_born_rad", a8::reflect::ET_INT32, my_offsetof2(MapThing, summon_born_rad_));
        meta_class->SetSimpleField(35, "shapes", a8::reflect::ET_STRING, my_offsetof2(MapThing, shapes_));
        meta_class->SetSimpleField(36, "delay_destroy", a8::reflect::ET_INT32, my_offsetof2(MapThing, delay_destroy_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> SafeArea::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("SafeArea", 11, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(SafeArea, id_));
        meta_class->SetSimpleField(1, "level", a8::reflect::ET_INT32, my_offsetof2(SafeArea, level_));
        meta_class->SetSimpleField(2, "rad", a8::reflect::ET_INT32, my_offsetof2(SafeArea, rad_));
        meta_class->SetSimpleField(3, "wait_time", a8::reflect::ET_INT32, my_offsetof2(SafeArea, wait_time_));
        meta_class->SetSimpleField(4, "shrink_speed", a8::reflect::ET_INT32, my_offsetof2(SafeArea, shrink_speed_));
        meta_class->SetSimpleField(5, "hurt", a8::reflect::ET_FLOAT, my_offsetof2(SafeArea, hurt_));
        meta_class->SetSimpleField(6, "type", a8::reflect::ET_INT32, my_offsetof2(SafeArea, type_));
        meta_class->SetSimpleField(7, "x1", a8::reflect::ET_INT32, my_offsetof2(SafeArea, x1_));
        meta_class->SetSimpleField(8, "y1", a8::reflect::ET_INT32, my_offsetof2(SafeArea, y1_));
        meta_class->SetSimpleField(9, "boss", a8::reflect::ET_STRING, my_offsetof2(SafeArea, boss_));
        meta_class->SetSimpleField(10, "rebirth", a8::reflect::ET_INT32, my_offsetof2(SafeArea, rebirth_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> SafeAreaSafePoint::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("SafeAreaSafePoint", 5, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(SafeAreaSafePoint, id_));
        meta_class->SetSimpleField(1, "type", a8::reflect::ET_INT32, my_offsetof2(SafeAreaSafePoint, type_));
        meta_class->SetSimpleField(2, "x", a8::reflect::ET_FLOAT, my_offsetof2(SafeAreaSafePoint, x_));
        meta_class->SetSimpleField(3, "y", a8::reflect::ET_FLOAT, my_offsetof2(SafeAreaSafePoint, y_));
        meta_class->SetSimpleField(4, "z", a8::reflect::ET_FLOAT, my_offsetof2(SafeAreaSafePoint, z_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> SafeAreaPos::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("SafeAreaPos", 21, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, id_));
        meta_class->SetSimpleField(1, "x1", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x1_));
        meta_class->SetSimpleField(2, "y1", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y1_));
        meta_class->SetSimpleField(3, "x2", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x2_));
        meta_class->SetSimpleField(4, "y2", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y2_));
        meta_class->SetSimpleField(5, "x3", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x3_));
        meta_class->SetSimpleField(6, "y3", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y3_));
        meta_class->SetSimpleField(7, "x4", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x4_));
        meta_class->SetSimpleField(8, "y4", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y4_));
        meta_class->SetSimpleField(9, "x5", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x5_));
        meta_class->SetSimpleField(10, "y5", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y5_));
        meta_class->SetSimpleField(11, "x6", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x6_));
        meta_class->SetSimpleField(12, "y6", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y6_));
        meta_class->SetSimpleField(13, "x7", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x7_));
        meta_class->SetSimpleField(14, "y7", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y7_));
        meta_class->SetSimpleField(15, "x8", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x8_));
        meta_class->SetSimpleField(16, "y8", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y8_));
        meta_class->SetSimpleField(17, "x9", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x9_));
        meta_class->SetSimpleField(18, "y9", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y9_));
        meta_class->SetSimpleField(19, "x10", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, x10_));
        meta_class->SetSimpleField(20, "y10", a8::reflect::ET_INT32, my_offsetof2(SafeAreaPos, y10_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Item::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Item", 10, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Item, id_));
        meta_class->SetSimpleField(1, "type", a8::reflect::ET_INT32, my_offsetof2(Item, type_));
        meta_class->SetSimpleField(2, "sub_type", a8::reflect::ET_INT32, my_offsetof2(Item, sub_type_));
        meta_class->SetSimpleField(3, "quality", a8::reflect::ET_INT32, my_offsetof2(Item, quality_));
        meta_class->SetSimpleField(4, "use", a8::reflect::ET_INT32, my_offsetof2(Item, use_));
        meta_class->SetSimpleField(5, "skinid", a8::reflect::ET_INT32, my_offsetof2(Item, skinid_));
        meta_class->SetSimpleField(6, "isdefaultskin", a8::reflect::ET_INT32, my_offsetof2(Item, isdefaultskin_));
        meta_class->SetSimpleField(7, "playerid", a8::reflect::ET_INT32, my_offsetof2(Item, playerid_));
        meta_class->SetSimpleField(8, "relationship", a8::reflect::ET_INT32, my_offsetof2(Item, relationship_));
        meta_class->SetSimpleField(9, "name", a8::reflect::ET_STRING, my_offsetof2(Item, name_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Equip::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Equip", 66, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Equip, id_));
        meta_class->SetSimpleField(1, "equip_type", a8::reflect::ET_INT32, my_offsetof2(Equip, equip_type_));
        meta_class->SetSimpleField(2, "equip_subtype", a8::reflect::ET_INT32, my_offsetof2(Equip, equip_subtype_));
        meta_class->SetSimpleField(3, "equip_lv", a8::reflect::ET_INT32, my_offsetof2(Equip, equip_lv_));
        meta_class->SetSimpleField(4, "fire_mode", a8::reflect::ET_INT32, my_offsetof2(Equip, fire_mode_));
        meta_class->SetSimpleField(5, "use_bullet", a8::reflect::ET_INT32, my_offsetof2(Equip, use_bullet_));
        meta_class->SetSimpleField(6, "clip_volume", a8::reflect::ET_INT32, my_offsetof2(Equip, clip_volume_));
        meta_class->SetSimpleField(7, "reload_time", a8::reflect::ET_INT32, my_offsetof2(Equip, reload_time_));
        meta_class->SetSimpleField(8, "fire_rate", a8::reflect::ET_INT32, my_offsetof2(Equip, fire_rate_));
        meta_class->SetSimpleField(9, "atk", a8::reflect::ET_STRING, my_offsetof2(Equip, atk_));
        meta_class->SetSimpleField(10, "def", a8::reflect::ET_INT32, my_offsetof2(Equip, def_));
        meta_class->SetSimpleField(11, "max_hp", a8::reflect::ET_STRING, my_offsetof2(Equip, max_hp_));
        meta_class->SetSimpleField(12, "explosion_range", a8::reflect::ET_INT32, my_offsetof2(Equip, explosion_range_));
        meta_class->SetSimpleField(13, "bullet_speed", a8::reflect::ET_INT32, my_offsetof2(Equip, bullet_speed_));
        meta_class->SetSimpleField(14, "range", a8::reflect::ET_INT32, my_offsetof2(Equip, range_));
        meta_class->SetSimpleField(15, "use_time", a8::reflect::ET_INT32, my_offsetof2(Equip, use_time_));
        meta_class->SetSimpleField(16, "heal", a8::reflect::ET_STRING, my_offsetof2(Equip, heal_));
        meta_class->SetSimpleField(17, "time", a8::reflect::ET_INT32, my_offsetof2(Equip, time_));
        meta_class->SetSimpleField(18, "volume", a8::reflect::ET_STRING, my_offsetof2(Equip, volume_));
        meta_class->SetSimpleField(19, "bullet_rad", a8::reflect::ET_INT32, my_offsetof2(Equip, bullet_rad_));
        meta_class->SetSimpleField(20, "group_num", a8::reflect::ET_INT32, my_offsetof2(Equip, group_num_));
        meta_class->SetSimpleField(21, "is_luck", a8::reflect::ET_INT32, my_offsetof2(Equip, is_luck_));
        meta_class->SetSimpleField(22, "bullet_born_offset", a8::reflect::ET_STRING, my_offsetof2(Equip, bullet_born_offset_));
        meta_class->SetSimpleField(23, "bullet_angle", a8::reflect::ET_FLOAT, my_offsetof2(Equip, bullet_angle_));
        meta_class->SetSimpleField(24, "name", a8::reflect::ET_STRING, my_offsetof2(Equip, name_));
        meta_class->SetSimpleField(25, "rad", a8::reflect::ET_FLOAT, my_offsetof2(Equip, rad_));
        meta_class->SetSimpleField(26, "rad2", a8::reflect::ET_FLOAT, my_offsetof2(Equip, rad2_));
        meta_class->SetSimpleField(27, "buffid", a8::reflect::ET_INT32, my_offsetof2(Equip, buffid_));
        meta_class->SetSimpleField(28, "drop_id", a8::reflect::ET_INT32, my_offsetof2(Equip, drop_id_));
        meta_class->SetSimpleField(29, "explosion_effect", a8::reflect::ET_INT32, my_offsetof2(Equip, explosion_effect_));
        meta_class->SetSimpleField(30, "param1", a8::reflect::ET_STRING, my_offsetof2(Equip, param1_));
        meta_class->SetSimpleField(31, "param2", a8::reflect::ET_STRING, my_offsetof2(Equip, param2_));
        meta_class->SetSimpleField(32, "reloadtype", a8::reflect::ET_INT32, my_offsetof2(Equip, reloadtype_));
        meta_class->SetSimpleField(33, "recoil_force", a8::reflect::ET_FLOAT, my_offsetof2(Equip, recoil_force_));
        meta_class->SetSimpleField(34, "missiles_time", a8::reflect::ET_INT32, my_offsetof2(Equip, missiles_time_));
        meta_class->SetSimpleField(35, "heroid", a8::reflect::ET_INT32, my_offsetof2(Equip, heroid_));
        meta_class->SetSimpleField(36, "launch_dev", a8::reflect::ET_STRING, my_offsetof2(Equip, launch_dev_));
        meta_class->SetSimpleField(37, "power_time", a8::reflect::ET_STRING, my_offsetof2(Equip, power_time_));
        meta_class->SetSimpleField(38, "power_gun", a8::reflect::ET_STRING, my_offsetof2(Equip, power_gun_));
        meta_class->SetSimpleField(39, "power_buff", a8::reflect::ET_STRING, my_offsetof2(Equip, power_buff_));
        meta_class->SetSimpleField(40, "through_teammate", a8::reflect::ET_INT32, my_offsetof2(Equip, through_teammate_));
        meta_class->SetSimpleField(41, "text_icon", a8::reflect::ET_STRING, my_offsetof2(Equip, text_icon_));
        meta_class->SetSimpleField(42, "special_damage_type", a8::reflect::ET_STRING, my_offsetof2(Equip, special_damage_type_));
        meta_class->SetSimpleField(43, "max_oil", a8::reflect::ET_FLOAT, my_offsetof2(Equip, max_oil_));
        meta_class->SetSimpleField(44, "average_oil", a8::reflect::ET_FLOAT, my_offsetof2(Equip, average_oil_));
        meta_class->SetSimpleField(45, "atk_mech", a8::reflect::ET_FLOAT, my_offsetof2(Equip, atk_mech_));
        meta_class->SetSimpleField(46, "use_scene", a8::reflect::ET_INT32, my_offsetof2(Equip, use_scene_));
        meta_class->SetSimpleField(47, "ispenetrate", a8::reflect::ET_INT32, my_offsetof2(Equip, ispenetrate_));
        meta_class->SetSimpleField(48, "is_penetrate_thing", a8::reflect::ET_INT32, my_offsetof2(Equip, is_penetrate_thing_));
        meta_class->SetSimpleField(49, "reload_delay_time", a8::reflect::ET_INT32, my_offsetof2(Equip, reload_delay_time_));
        meta_class->SetSimpleField(50, "cast_time", a8::reflect::ET_INT32, my_offsetof2(Equip, cast_time_));
        meta_class->SetSimpleField(51, "aiming_cast_time", a8::reflect::ET_INT32, my_offsetof2(Equip, aiming_cast_time_));
        meta_class->SetSimpleField(52, "auto_switch_weapon_time", a8::reflect::ET_INT32, my_offsetof2(Equip, auto_switch_weapon_time_));
        meta_class->SetSimpleField(53, "quality", a8::reflect::ET_INT32, my_offsetof2(Equip, quality_));
        meta_class->SetSimpleField(54, "explosion_damage_delay", a8::reflect::ET_INT32, my_offsetof2(Equip, explosion_damage_delay_));
        meta_class->SetSimpleField(55, "bullet_consume_type", a8::reflect::ET_INT32, my_offsetof2(Equip, bullet_consume_type_));
        meta_class->SetSimpleField(56, "inventory_slot", a8::reflect::ET_STRING, my_offsetof2(Equip, inventory_slot_));
        meta_class->SetSimpleField(57, "_inventory_slot", a8::reflect::ET_INT32, my_offsetof2(Equip, _inventory_slot_));
        meta_class->SetSimpleField(58, "critical", a8::reflect::ET_FLOAT, my_offsetof2(Equip, critical_));
        meta_class->SetSimpleField(59, "cri_damage", a8::reflect::ET_FLOAT, my_offsetof2(Equip, cri_damage_));
        meta_class->SetSimpleField(60, "shootfire", a8::reflect::ET_INT32, my_offsetof2(Equip, shootfire_));
        meta_class->SetSimpleField(61, "hit_buff", a8::reflect::ET_STRING, my_offsetof2(Equip, hit_buff_));
        meta_class->SetSimpleField(62, "auto_trace", a8::reflect::ET_INT32, my_offsetof2(Equip, auto_trace_));
        meta_class->SetSimpleField(63, "trace_range", a8::reflect::ET_INT32, my_offsetof2(Equip, trace_range_));
        meta_class->SetSimpleField(64, "double_gun", a8::reflect::ET_INT32, my_offsetof2(Equip, double_gun_));
        meta_class->SetSimpleField(65, "hand", a8::reflect::ET_INT32, my_offsetof2(Equip, hand_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Hero::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Hero", 44, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Hero, id_));
        meta_class->SetSimpleField(1, "radius", a8::reflect::ET_FLOAT, my_offsetof2(Hero, radius_));
        meta_class->SetSimpleField(2, "move_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, move_speed_));
        meta_class->SetSimpleField(3, "jump_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, jump_speed_));
        meta_class->SetSimpleField(4, "fall_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, fall_speed_));
        meta_class->SetSimpleField(5, "aim_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, aim_speed_));
        meta_class->SetSimpleField(6, "shoot_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, shoot_speed_));
        meta_class->SetSimpleField(7, "reload_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, reload_speed_));
        meta_class->SetSimpleField(8, "medicine_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, medicine_speed_));
        meta_class->SetSimpleField(9, "swim_speed", a8::reflect::ET_FLOAT, my_offsetof2(Hero, swim_speed_));
        meta_class->SetSimpleField(10, "volume", a8::reflect::ET_STRING, my_offsetof2(Hero, volume_));
        meta_class->SetSimpleField(11, "level", a8::reflect::ET_INT32, my_offsetof2(Hero, level_));
        meta_class->SetSimpleField(12, "race", a8::reflect::ET_INT32, my_offsetof2(Hero, race_));
        meta_class->SetSimpleField(13, "name", a8::reflect::ET_STRING, my_offsetof2(Hero, name_));
        meta_class->SetSimpleField(14, "hit_offset_x", a8::reflect::ET_FLOAT, my_offsetof2(Hero, hit_offset_x_));
        meta_class->SetSimpleField(15, "hit_offset_y", a8::reflect::ET_FLOAT, my_offsetof2(Hero, hit_offset_y_));
        meta_class->SetSimpleField(16, "hit_radius", a8::reflect::ET_FLOAT, my_offsetof2(Hero, hit_radius_));
        meta_class->SetSimpleField(17, "ai_script", a8::reflect::ET_STRING, my_offsetof2(Hero, ai_script_));
        meta_class->SetSimpleField(18, "init_buffs", a8::reflect::ET_STRING, my_offsetof2(Hero, init_buffs_));
        meta_class->SetSimpleField(19, "default_weapon", a8::reflect::ET_INT32, my_offsetof2(Hero, default_weapon_));
        meta_class->SetSimpleField(20, "dead_drop", a8::reflect::ET_STRING, my_offsetof2(Hero, dead_drop_));
        meta_class->SetSimpleField(21, "delay_delete", a8::reflect::ET_INT32, my_offsetof2(Hero, delay_delete_));
        meta_class->SetSimpleField(22, "ai", a8::reflect::ET_INT32, my_offsetof2(Hero, ai_));
        meta_class->SetSimpleField(23, "delay_remove", a8::reflect::ET_INT32, my_offsetof2(Hero, delay_remove_));
        meta_class->SetSimpleField(24, "skinlist", a8::reflect::ET_INT32, my_offsetof2(Hero, skinlist_));
        meta_class->SetSimpleField(25, "pre_appear_effect", a8::reflect::ET_STRING, my_offsetof2(Hero, pre_appear_effect_));
        meta_class->SetSimpleField(26, "pve_score", a8::reflect::ET_STRING, my_offsetof2(Hero, pve_score_));
        meta_class->SetSimpleField(27, "hp", a8::reflect::ET_INT32, my_offsetof2(Hero, hp_));
        meta_class->SetSimpleField(28, "damage", a8::reflect::ET_INT32, my_offsetof2(Hero, damage_));
        meta_class->SetSimpleField(29, "defence", a8::reflect::ET_INT32, my_offsetof2(Hero, defence_));
        meta_class->SetSimpleField(30, "crit_atk", a8::reflect::ET_FLOAT, my_offsetof2(Hero, crit_atk_));
        meta_class->SetSimpleField(31, "crit_atk_ratio", a8::reflect::ET_FLOAT, my_offsetof2(Hero, crit_atk_ratio_));
        meta_class->SetSimpleField(32, "miss", a8::reflect::ET_FLOAT, my_offsetof2(Hero, miss_));
        meta_class->SetSimpleField(33, "miss_damage_ruduce", a8::reflect::ET_FLOAT, my_offsetof2(Hero, miss_damage_ruduce_));
        meta_class->SetSimpleField(34, "skill1list", a8::reflect::ET_INT32, my_offsetof2(Hero, skill1list_));
        meta_class->SetSimpleField(35, "skill2list", a8::reflect::ET_INT32, my_offsetof2(Hero, skill2list_));
        meta_class->SetSimpleField(36, "skill3list", a8::reflect::ET_INT32, my_offsetof2(Hero, skill3list_));
        meta_class->SetSimpleField(37, "hp_ratio", a8::reflect::ET_FLOAT, my_offsetof2(Hero, hp_ratio_));
        meta_class->SetSimpleField(38, "damage_ratio", a8::reflect::ET_FLOAT, my_offsetof2(Hero, damage_ratio_));
        meta_class->SetSimpleField(39, "defence_ratio", a8::reflect::ET_FLOAT, my_offsetof2(Hero, defence_ratio_));
        meta_class->SetSimpleField(40, "bt", a8::reflect::ET_STRING, my_offsetof2(Hero, bt_));
        meta_class->SetSimpleField(41, "drop", a8::reflect::ET_STRING, my_offsetof2(Hero, drop_));
        meta_class->SetSimpleField(42, "new_bt", a8::reflect::ET_STRING, my_offsetof2(Hero, new_bt_));
        meta_class->SetSimpleField(43, "other_key_time", a8::reflect::ET_STRING, my_offsetof2(Hero, other_key_time_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Robot::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Robot", 9, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Robot, id_));
        meta_class->SetSimpleField(1, "name", a8::reflect::ET_STRING, my_offsetof2(Robot, name_));
        meta_class->SetSimpleField(2, "hero_id", a8::reflect::ET_INT32, my_offsetof2(Robot, hero_id_));
        meta_class->SetSimpleField(3, "skin_id", a8::reflect::ET_STRING, my_offsetof2(Robot, skin_id_));
        meta_class->SetSimpleField(4, "weapon_id", a8::reflect::ET_INT32, my_offsetof2(Robot, weapon_id_));
        meta_class->SetSimpleField(5, "weapon_lv", a8::reflect::ET_INT32, my_offsetof2(Robot, weapon_lv_));
        meta_class->SetSimpleField(6, "sex", a8::reflect::ET_INT32, my_offsetof2(Robot, sex_));
        meta_class->SetSimpleField(7, "dmg_ratio", a8::reflect::ET_FLOAT, my_offsetof2(Robot, dmg_ratio_));
        meta_class->SetSimpleField(8, "bullet_offset", a8::reflect::ET_STRING, my_offsetof2(Robot, bullet_offset_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Skill::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Skill", 29, 0);
        meta_class->SetSimpleField(0, "skill_id", a8::reflect::ET_INT32, my_offsetof2(Skill, skill_id_));
        meta_class->SetSimpleField(1, "skill_type", a8::reflect::ET_INT32, my_offsetof2(Skill, skill_type_));
        meta_class->SetSimpleField(2, "value_up", a8::reflect::ET_STRING, my_offsetof2(Skill, value_up_));
        meta_class->SetSimpleField(3, "skill_cd", a8::reflect::ET_INT32, my_offsetof2(Skill, skill_cd_));
        meta_class->SetSimpleField(4, "skill_target", a8::reflect::ET_INT32, my_offsetof2(Skill, skill_target_));
        meta_class->SetSimpleField(5, "buff_list", a8::reflect::ET_STRING, my_offsetof2(Skill, buff_list_));
        meta_class->SetSimpleField(6, "skill_distance", a8::reflect::ET_FLOAT, my_offsetof2(Skill, skill_distance_));
        meta_class->SetSimpleField(7, "cold_time_up", a8::reflect::ET_INT32, my_offsetof2(Skill, cold_time_up_));
        meta_class->SetSimpleField(8, "max_times", a8::reflect::ET_INT32, my_offsetof2(Skill, max_times_));
        meta_class->SetSimpleField(9, "phase1_time_offset", a8::reflect::ET_INT32, my_offsetof2(Skill, phase1_time_offset_));
        meta_class->SetSimpleField(10, "phase1_func", a8::reflect::ET_INT32, my_offsetof2(Skill, phase1_func_));
        meta_class->SetSimpleField(11, "phase1_param1", a8::reflect::ET_STRING, my_offsetof2(Skill, phase1_param1_));
        meta_class->SetSimpleField(12, "phase1_param2", a8::reflect::ET_STRING, my_offsetof2(Skill, phase1_param2_));
        meta_class->SetSimpleField(13, "phase1_param3", a8::reflect::ET_STRING, my_offsetof2(Skill, phase1_param3_));
        meta_class->SetSimpleField(14, "phase2_time_offset", a8::reflect::ET_INT32, my_offsetof2(Skill, phase2_time_offset_));
        meta_class->SetSimpleField(15, "phase2_func", a8::reflect::ET_INT32, my_offsetof2(Skill, phase2_func_));
        meta_class->SetSimpleField(16, "phase2_param1", a8::reflect::ET_STRING, my_offsetof2(Skill, phase2_param1_));
        meta_class->SetSimpleField(17, "phase2_param2", a8::reflect::ET_STRING, my_offsetof2(Skill, phase2_param2_));
        meta_class->SetSimpleField(18, "phase2_param3", a8::reflect::ET_STRING, my_offsetof2(Skill, phase2_param3_));
        meta_class->SetSimpleField(19, "phase3_time_offset", a8::reflect::ET_INT32, my_offsetof2(Skill, phase3_time_offset_));
        meta_class->SetSimpleField(20, "phase3_func", a8::reflect::ET_INT32, my_offsetof2(Skill, phase3_func_));
        meta_class->SetSimpleField(21, "phase3_param1", a8::reflect::ET_STRING, my_offsetof2(Skill, phase3_param1_));
        meta_class->SetSimpleField(22, "phase3_param2", a8::reflect::ET_STRING, my_offsetof2(Skill, phase3_param2_));
        meta_class->SetSimpleField(23, "phase3_param3", a8::reflect::ET_STRING, my_offsetof2(Skill, phase3_param3_));
        meta_class->SetSimpleField(24, "cast_time", a8::reflect::ET_INT32, my_offsetof2(Skill, cast_time_));
        meta_class->SetSimpleField(25, "up_exp", a8::reflect::ET_INT32, my_offsetof2(Skill, up_exp_));
        meta_class->SetSimpleField(26, "nextlv_skill", a8::reflect::ET_INT32, my_offsetof2(Skill, nextlv_skill_));
        meta_class->SetSimpleField(27, "attack_dir_lock_time", a8::reflect::ET_INT32, my_offsetof2(Skill, attack_dir_lock_time_));
        meta_class->SetSimpleField(28, "skill_hold_buff", a8::reflect::ET_INT32, my_offsetof2(Skill, skill_hold_buff_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> SkillNumber::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("SkillNumber", 29, 0);
        meta_class->SetSimpleField(0, "skill_id", a8::reflect::ET_INT32, my_offsetof2(SkillNumber, skill_id_));
        meta_class->SetSimpleField(1, "skill_type", a8::reflect::ET_INT32, my_offsetof2(SkillNumber, skill_type_));
        meta_class->SetSimpleField(2, "number", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, number_));
        meta_class->SetSimpleField(3, "damage", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, damage_));
        meta_class->SetSimpleField(4, "damage_addition", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, damage_addition_));
        meta_class->SetSimpleField(5, "damage_change", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, damage_change_));
        meta_class->SetSimpleField(6, "shield", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, shield_));
        meta_class->SetSimpleField(7, "shield_addition", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, shield_addition_));
        meta_class->SetSimpleField(8, "resume", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, resume_));
        meta_class->SetSimpleField(9, "resume_addition", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, resume_addition_));
        meta_class->SetSimpleField(10, "speed", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, speed_));
        meta_class->SetSimpleField(11, "range", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range_));
        meta_class->SetSimpleField(12, "range2", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range2_));
        meta_class->SetSimpleField(13, "range3", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range3_));
        meta_class->SetSimpleField(14, "range4", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range4_));
        meta_class->SetSimpleField(15, "range5", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range5_));
        meta_class->SetSimpleField(16, "range6", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, range6_));
        meta_class->SetSimpleField(17, "time", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time_));
        meta_class->SetSimpleField(18, "cd", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, cd_));
        meta_class->SetSimpleField(19, "buff_id", a8::reflect::ET_INT32, my_offsetof2(SkillNumber, buff_id_));
        meta_class->SetSimpleField(20, "buff_time", a8::reflect::ET_FLOAT, my_offsetof2(SkillNumber, buff_time_));
        meta_class->SetSimpleField(21, "probability", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, probability_));
        meta_class->SetSimpleField(22, "explosion_effect", a8::reflect::ET_INT32, my_offsetof2(SkillNumber, explosion_effect_));
        meta_class->SetSimpleField(23, "effect_list", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, effect_list_));
        meta_class->SetSimpleField(24, "time2", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time2_));
        meta_class->SetSimpleField(25, "time3", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time3_));
        meta_class->SetSimpleField(26, "time4", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time4_));
        meta_class->SetSimpleField(27, "time5", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time5_));
        meta_class->SetSimpleField(28, "time6", a8::reflect::ET_STRING, my_offsetof2(SkillNumber, time6_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> NpcStandard::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("NpcStandard", 7, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, id_));
        meta_class->SetSimpleField(1, "quality", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, quality_));
        meta_class->SetSimpleField(2, "hp", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, hp_));
        meta_class->SetSimpleField(3, "damage", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, damage_));
        meta_class->SetSimpleField(4, "defence", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, defence_));
        meta_class->SetSimpleField(5, "hero_id", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, hero_id_));
        meta_class->SetSimpleField(6, "level", a8::reflect::ET_INT32, my_offsetof2(NpcStandard, level_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Buff::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Buff", 38, 0);
        meta_class->SetSimpleField(0, "buff_id", a8::reflect::ET_INT32, my_offsetof2(Buff, buff_id_));
        meta_class->SetSimpleField(1, "name", a8::reflect::ET_STRING, my_offsetof2(Buff, name_));
        meta_class->SetSimpleField(2, "buff_target", a8::reflect::ET_INT32, my_offsetof2(Buff, buff_target_));
        meta_class->SetSimpleField(3, "buff_effect", a8::reflect::ET_INT32, my_offsetof2(Buff, buff_effect_));
        meta_class->SetSimpleField(4, "trigger_type", a8::reflect::ET_INT32, my_offsetof2(Buff, trigger_type_));
        meta_class->SetSimpleField(5, "buff_param1", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param1_));
        meta_class->SetSimpleField(6, "buff_param2", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param2_));
        meta_class->SetSimpleField(7, "buff_param3", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param3_));
        meta_class->SetSimpleField(8, "buff_param4", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param4_));
        meta_class->SetSimpleField(9, "buff_param5", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param5_));
        meta_class->SetSimpleField(10, "buff_param6", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param6_));
        meta_class->SetSimpleField(11, "buff_param7", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param7_));
        meta_class->SetSimpleField(12, "buff_param8", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_param8_));
        meta_class->SetSimpleField(13, "duration_time", a8::reflect::ET_STRING, my_offsetof2(Buff, duration_time_));
        meta_class->SetSimpleField(14, "buff_valueup", a8::reflect::ET_FLOAT, my_offsetof2(Buff, buff_valueup_));
        meta_class->SetSimpleField(15, "immune_buffeffect_list", a8::reflect::ET_STRING, my_offsetof2(Buff, immune_buffeffect_list_));
        meta_class->SetSimpleField(16, "post_remove_action", a8::reflect::ET_STRING, my_offsetof2(Buff, post_remove_action_));
        meta_class->SetSimpleField(17, "only_server", a8::reflect::ET_INT32, my_offsetof2(Buff, only_server_));
        meta_class->SetSimpleField(18, "only_self", a8::reflect::ET_INT32, my_offsetof2(Buff, only_self_));
        meta_class->SetSimpleField(19, "depend_effect", a8::reflect::ET_INT32, my_offsetof2(Buff, depend_effect_));
        meta_class->SetSimpleField(20, "child_buff", a8::reflect::ET_STRING, my_offsetof2(Buff, child_buff_));
        meta_class->SetSimpleField(21, "coexist_num", a8::reflect::ET_INT32, my_offsetof2(Buff, coexist_num_));
        meta_class->SetSimpleField(22, "dead_valid", a8::reflect::ET_INT32, my_offsetof2(Buff, dead_valid_));
        meta_class->SetSimpleField(23, "buff_interval", a8::reflect::ET_STRING, my_offsetof2(Buff, buff_interval_));
        meta_class->SetSimpleField(24, "tag", a8::reflect::ET_STRING, my_offsetof2(Buff, tag_));
        meta_class->SetSimpleField(25, "post_battle_valid", a8::reflect::ET_INT32, my_offsetof2(Buff, post_battle_valid_));
        meta_class->SetSimpleField(26, "only_spec_race", a8::reflect::ET_STRING, my_offsetof2(Buff, only_spec_race_));
        meta_class->SetSimpleField(27, "exclude_spec_race", a8::reflect::ET_STRING, my_offsetof2(Buff, exclude_spec_race_));
        meta_class->SetSimpleField(28, "dead_remove", a8::reflect::ET_INT32, my_offsetof2(Buff, dead_remove_));
        meta_class->SetSimpleField(29, "no_immune", a8::reflect::ET_INT32, my_offsetof2(Buff, no_immune_));
        meta_class->SetSimpleField(30, "lock_move", a8::reflect::ET_INT32, my_offsetof2(Buff, lock_move_));
        meta_class->SetSimpleField(31, "lock_dir", a8::reflect::ET_INT32, my_offsetof2(Buff, lock_dir_));
        meta_class->SetSimpleField(32, "lock_move_dir", a8::reflect::ET_INT32, my_offsetof2(Buff, lock_move_dir_));
        meta_class->SetSimpleField(33, "disable_shot", a8::reflect::ET_INT32, my_offsetof2(Buff, disable_shot_));
        meta_class->SetSimpleField(34, "disable_useskill", a8::reflect::ET_INT32, my_offsetof2(Buff, disable_useskill_));
        meta_class->SetSimpleField(35, "effect_list", a8::reflect::ET_STRING, my_offsetof2(Buff, effect_list_));
        meta_class->SetSimpleField(36, "res_scale", a8::reflect::ET_STRING, my_offsetof2(Buff, res_scale_));
        meta_class->SetSimpleField(37, "tenacity", a8::reflect::ET_INT32, my_offsetof2(Buff, tenacity_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Drop::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Drop", 5, 0);
        meta_class->SetSimpleField(0, "drop_id", a8::reflect::ET_INT32, my_offsetof2(Drop, drop_id_));
        meta_class->SetSimpleField(1, "item_id", a8::reflect::ET_STRING, my_offsetof2(Drop, item_id_));
        meta_class->SetSimpleField(2, "num", a8::reflect::ET_STRING, my_offsetof2(Drop, num_));
        meta_class->SetSimpleField(3, "weight", a8::reflect::ET_STRING, my_offsetof2(Drop, weight_));
        meta_class->SetSimpleField(4, "type", a8::reflect::ET_INT32, my_offsetof2(Drop, type_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> AirDrop::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("AirDrop", 4, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(AirDrop, id_));
        meta_class->SetSimpleField(1, "time", a8::reflect::ET_INT32, my_offsetof2(AirDrop, time_));
        meta_class->SetSimpleField(2, "appear_time", a8::reflect::ET_INT32, my_offsetof2(AirDrop, appear_time_));
        meta_class->SetSimpleField(3, "drop_id", a8::reflect::ET_STRING, my_offsetof2(AirDrop, drop_id_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> AirRaid::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("AirRaid", 8, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(AirRaid, id_));
        meta_class->SetSimpleField(1, "time", a8::reflect::ET_INT32, my_offsetof2(AirRaid, time_));
        meta_class->SetSimpleField(2, "appear_time", a8::reflect::ET_INT32, my_offsetof2(AirRaid, appear_time_));
        meta_class->SetSimpleField(3, "bomb_id", a8::reflect::ET_STRING, my_offsetof2(AirRaid, bomb_id_));
        meta_class->SetSimpleField(4, "raid_wave", a8::reflect::ET_STRING, my_offsetof2(AirRaid, raid_wave_));
        meta_class->SetSimpleField(5, "rad", a8::reflect::ET_FLOAT, my_offsetof2(AirRaid, rad_));
        meta_class->SetSimpleField(6, "bombling_time", a8::reflect::ET_STRING, my_offsetof2(AirRaid, bombling_time_));
        meta_class->SetSimpleField(7, "damage", a8::reflect::ET_FLOAT, my_offsetof2(AirRaid, damage_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> AirLine::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("AirLine", 6, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(AirLine, id_));
        meta_class->SetSimpleField(1, "start_point", a8::reflect::ET_STRING, my_offsetof2(AirLine, start_point_));
        meta_class->SetSimpleField(2, "end_point", a8::reflect::ET_STRING, my_offsetof2(AirLine, end_point_));
        meta_class->SetSimpleField(3, "plane_speed", a8::reflect::ET_FLOAT, my_offsetof2(AirLine, plane_speed_));
        meta_class->SetSimpleField(4, "weight", a8::reflect::ET_INT32, my_offsetof2(AirLine, weight_));
        meta_class->SetSimpleField(5, "map_id", a8::reflect::ET_INT32, my_offsetof2(AirLine, map_id_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> RankReward::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("RankReward", 3, 0);
        meta_class->SetSimpleField(0, "rank", a8::reflect::ET_INT32, my_offsetof2(RankReward, rank_));
        meta_class->SetSimpleField(1, "parameter", a8::reflect::ET_FLOAT, my_offsetof2(RankReward, parameter_));
        meta_class->SetSimpleField(2, "drop", a8::reflect::ET_INT32, my_offsetof2(RankReward, drop_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> RankPoint::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("RankPoint", 4, 0);
        meta_class->SetSimpleField(0, "rank", a8::reflect::ET_INT32, my_offsetof2(RankPoint, rank_));
        meta_class->SetSimpleField(1, "parameter", a8::reflect::ET_INT32, my_offsetof2(RankPoint, parameter_));
        meta_class->SetSimpleField(2, "parameter2", a8::reflect::ET_INT32, my_offsetof2(RankPoint, parameter2_));
        meta_class->SetSimpleField(3, "parameter3", a8::reflect::ET_INT32, my_offsetof2(RankPoint, parameter3_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> KillReward::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("KillReward", 2, 0);
        meta_class->SetSimpleField(0, "kill_num", a8::reflect::ET_INT32, my_offsetof2(KillReward, kill_num_));
        meta_class->SetSimpleField(1, "parameter", a8::reflect::ET_FLOAT, my_offsetof2(KillReward, parameter_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> KillPoint::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("KillPoint", 3, 0);
        meta_class->SetSimpleField(0, "kill_num", a8::reflect::ET_INT32, my_offsetof2(KillPoint, kill_num_));
        meta_class->SetSimpleField(1, "parameter", a8::reflect::ET_INT32, my_offsetof2(KillPoint, parameter_));
        meta_class->SetSimpleField(2, "parameter2", a8::reflect::ET_INT32, my_offsetof2(KillPoint, parameter2_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> AI::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("AI", 18, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(AI, id_));
        meta_class->SetSimpleField(1, "pursuit_radius", a8::reflect::ET_INT32, my_offsetof2(AI, pursuit_radius_));
        meta_class->SetSimpleField(2, "attack_interval", a8::reflect::ET_INT32, my_offsetof2(AI, attack_interval_));
        meta_class->SetSimpleField(3, "attack_times", a8::reflect::ET_INT32, my_offsetof2(AI, attack_times_));
        meta_class->SetSimpleField(4, "attack_type", a8::reflect::ET_INT32, my_offsetof2(AI, attack_type_));
        meta_class->SetSimpleField(5, "shot_offset_angle", a8::reflect::ET_INT32, my_offsetof2(AI, shot_offset_angle_));
        meta_class->SetSimpleField(6, "random_move_idle_time", a8::reflect::ET_STRING, my_offsetof2(AI, random_move_idle_time_));
        meta_class->SetSimpleField(7, "random_move_time", a8::reflect::ET_STRING, my_offsetof2(AI, random_move_time_));
        meta_class->SetSimpleField(8, "attack_range", a8::reflect::ET_INT32, my_offsetof2(AI, attack_range_));
        meta_class->SetSimpleField(9, "attack_rate", a8::reflect::ET_FLOAT, my_offsetof2(AI, attack_rate_));
        meta_class->SetSimpleField(10, "ai_mode", a8::reflect::ET_INT32, my_offsetof2(AI, ai_mode_));
        meta_class->SetSimpleField(11, "ai_kind", a8::reflect::ET_INT32, my_offsetof2(AI, ai_kind_));
        meta_class->SetSimpleField(12, "param1", a8::reflect::ET_STRING, my_offsetof2(AI, param1_));
        meta_class->SetSimpleField(13, "param2", a8::reflect::ET_STRING, my_offsetof2(AI, param2_));
        meta_class->SetSimpleField(14, "param3", a8::reflect::ET_STRING, my_offsetof2(AI, param3_));
        meta_class->SetSimpleField(15, "param4", a8::reflect::ET_STRING, my_offsetof2(AI, param4_));
        meta_class->SetSimpleField(16, "param5", a8::reflect::ET_STRING, my_offsetof2(AI, param5_));
        meta_class->SetSimpleField(17, "peace_time", a8::reflect::ET_INT32, my_offsetof2(AI, peace_time_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Text::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Text", 2, 0);
        meta_class->SetSimpleField(0, "textid", a8::reflect::ET_STRING, my_offsetof2(Text, textid_));
        meta_class->SetSimpleField(1, "text", a8::reflect::ET_STRING, my_offsetof2(Text, text_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> HeroQuality::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("HeroQuality", 4, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(HeroQuality, id_));
        meta_class->SetSimpleField(1, "quality", a8::reflect::ET_INT32, my_offsetof2(HeroQuality, quality_));
        meta_class->SetSimpleField(2, "gold_limit", a8::reflect::ET_INT32, my_offsetof2(HeroQuality, gold_limit_));
        meta_class->SetSimpleField(3, "lucky", a8::reflect::ET_INT32, my_offsetof2(HeroQuality, lucky_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> GunQuality::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("GunQuality", 4, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(GunQuality, id_));
        meta_class->SetSimpleField(1, "quality", a8::reflect::ET_INT32, my_offsetof2(GunQuality, quality_));
        meta_class->SetSimpleField(2, "gold_limit", a8::reflect::ET_INT32, my_offsetof2(GunQuality, gold_limit_));
        meta_class->SetSimpleField(3, "lucky", a8::reflect::ET_INT32, my_offsetof2(GunQuality, lucky_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> FormulaPvp::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("FormulaPvp", 6, 0);
        meta_class->SetSimpleField(0, "top", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, top_));
        meta_class->SetSimpleField(1, "ranked_topx", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, ranked_topx_));
        meta_class->SetSimpleField(2, "kills_topx", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, kills_topx_));
        meta_class->SetSimpleField(3, "hero_topx", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, hero_topx_));
        meta_class->SetSimpleField(4, "weapon_topx", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, weapon_topx_));
        meta_class->SetSimpleField(5, "survival_topx", a8::reflect::ET_FLOAT, my_offsetof2(FormulaPvp, survival_topx_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> PveGemini::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("PveGemini", 5, 0);
        meta_class->SetSimpleField(0, "gemini_id", a8::reflect::ET_INT32, my_offsetof2(PveGemini, gemini_id_));
        meta_class->SetSimpleField(1, "gemini_lv", a8::reflect::ET_INT32, my_offsetof2(PveGemini, gemini_lv_));
        meta_class->SetSimpleField(2, "gemini_limit", a8::reflect::ET_INT32, my_offsetof2(PveGemini, gemini_limit_));
        meta_class->SetSimpleField(3, "multiplayer_enemy_hp_mul", a8::reflect::ET_STRING, my_offsetof2(PveGemini, multiplayer_enemy_hp_mul_));
        meta_class->SetSimpleField(4, "born_point", a8::reflect::ET_STRING, my_offsetof2(PveGemini, born_point_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> PveGeminiContent::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("PveGeminiContent", 5, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(PveGeminiContent, id_));
        meta_class->SetSimpleField(1, "mode_id", a8::reflect::ET_INT32, my_offsetof2(PveGeminiContent, mode_id_));
        meta_class->SetSimpleField(2, "round", a8::reflect::ET_INT32, my_offsetof2(PveGeminiContent, round_));
        meta_class->SetSimpleField(3, "spawn_point", a8::reflect::ET_STRING, my_offsetof2(PveGeminiContent, spawn_point_));
        meta_class->SetSimpleField(4, "enemy_id", a8::reflect::ET_STRING, my_offsetof2(PveGeminiContent, enemy_id_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> PveGeminiMode::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("PveGeminiMode", 8, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(PveGeminiMode, id_));
        meta_class->SetSimpleField(1, "map_id", a8::reflect::ET_INT32, my_offsetof2(PveGeminiMode, map_id_));
        meta_class->SetSimpleField(2, "mode_time", a8::reflect::ET_STRING, my_offsetof2(PveGeminiMode, mode_time_));
        meta_class->SetSimpleField(3, "area", a8::reflect::ET_STRING, my_offsetof2(PveGeminiMode, area_));
        meta_class->SetSimpleField(4, "score_reward", a8::reflect::ET_STRING, my_offsetof2(PveGeminiMode, score_reward_));
        meta_class->SetSimpleField(5, "round_score", a8::reflect::ET_STRING, my_offsetof2(PveGeminiMode, round_score_));
        meta_class->SetSimpleField(6, "next_door", a8::reflect::ET_STRING, my_offsetof2(PveGeminiMode, next_door_));
        meta_class->SetSimpleField(7, "wave_prepare_time", a8::reflect::ET_INT32, my_offsetof2(PveGeminiMode, wave_prepare_time_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> RankRoom::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("RankRoom", 10, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(RankRoom, id_));
        meta_class->SetSimpleField(1, "elo_min", a8::reflect::ET_INT32, my_offsetof2(RankRoom, elo_min_));
        meta_class->SetSimpleField(2, "elo_max", a8::reflect::ET_INT32, my_offsetof2(RankRoom, elo_max_));
        meta_class->SetSimpleField(3, "elo_expansion1", a8::reflect::ET_INT32, my_offsetof2(RankRoom, elo_expansion1_));
        meta_class->SetSimpleField(4, "elo_expansion2", a8::reflect::ET_INT32, my_offsetof2(RankRoom, elo_expansion2_));
        meta_class->SetSimpleField(5, "expand_time1", a8::reflect::ET_INT32, my_offsetof2(RankRoom, expand_time1_));
        meta_class->SetSimpleField(6, "expand_time2", a8::reflect::ET_INT32, my_offsetof2(RankRoom, expand_time2_));
        meta_class->SetSimpleField(7, "player_num_standard", a8::reflect::ET_INT32, my_offsetof2(RankRoom, player_num_standard_));
        meta_class->SetSimpleField(8, "final_time", a8::reflect::ET_INT32, my_offsetof2(RankRoom, final_time_));
        meta_class->SetSimpleField(9, "final_player_num", a8::reflect::ET_INT32, my_offsetof2(RankRoom, final_player_num_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> WorldObject::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("WorldObject", 5, 0);
        meta_class->SetSimpleField(0, "object_id", a8::reflect::ET_INT32, my_offsetof2(WorldObject, object_id_));
        meta_class->SetSimpleField(1, "object_type", a8::reflect::ET_INT32, my_offsetof2(WorldObject, object_type_));
        meta_class->SetSimpleField(2, "x", a8::reflect::ET_FLOAT, my_offsetof2(WorldObject, x_));
        meta_class->SetSimpleField(3, "y", a8::reflect::ET_FLOAT, my_offsetof2(WorldObject, y_));
        meta_class->SetSimpleField(4, "z", a8::reflect::ET_FLOAT, my_offsetof2(WorldObject, z_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MergeItem::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MergeItem", 5, 0);
        meta_class->SetSimpleField(0, "equip_id", a8::reflect::ET_INT32, my_offsetof2(MergeItem, equip_id_));
        meta_class->SetSimpleField(1, "skill_id", a8::reflect::ET_INT32, my_offsetof2(MergeItem, skill_id_));
        meta_class->SetSimpleField(2, "pickup3", a8::reflect::ET_STRING, my_offsetof2(MergeItem, pickup3_));
        meta_class->SetSimpleField(3, "pickup6", a8::reflect::ET_STRING, my_offsetof2(MergeItem, pickup6_));
        meta_class->SetSimpleField(4, "pickup9", a8::reflect::ET_STRING, my_offsetof2(MergeItem, pickup9_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> MapThingGroup::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("MapThingGroup", 4, 0);
        meta_class->SetSimpleField(0, "mtGroupId", a8::reflect::ET_INT32, my_offsetof2(MapThingGroup, mtGroupId_));
        meta_class->SetSimpleField(1, "rule", a8::reflect::ET_STRING, my_offsetof2(MapThingGroup, rule_));
        meta_class->SetSimpleField(2, "mapThings", a8::reflect::ET_STRING, my_offsetof2(MapThingGroup, mapThings_));
        meta_class->SetSimpleField(3, "heroes", a8::reflect::ET_STRING, my_offsetof2(MapThingGroup, heroes_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleBasicAttribute::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleBasicAttribute", 32, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleBasicAttribute, id_));
        meta_class->SetSimpleField(1, "pBaseHealth", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pBaseHealth_));
        meta_class->SetSimpleField(2, "pOrigHealth", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pOrigHealth_));
        meta_class->SetSimpleField(3, "vOrigHealth", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigHealth_));
        meta_class->SetSimpleField(4, "pBaseAttack", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pBaseAttack_));
        meta_class->SetSimpleField(5, "pOrigAttack", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pOrigAttack_));
        meta_class->SetSimpleField(6, "vOrigAttack", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigAttack_));
        meta_class->SetSimpleField(7, "pBaseDefend", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pBaseDefend_));
        meta_class->SetSimpleField(8, "pOrigDefend", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pOrigDefend_));
        meta_class->SetSimpleField(9, "vOrigDefend", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigDefend_));
        meta_class->SetSimpleField(10, "pBaseBlock", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pBaseBlock_));
        meta_class->SetSimpleField(11, "pOrigBlock", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pOrigBlock_));
        meta_class->SetSimpleField(12, "vOrigBlock", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigBlock_));
        meta_class->SetSimpleField(13, "vOrigEvaDef", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigEvaDef_));
        meta_class->SetSimpleField(14, "pBaseCrit", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pBaseCrit_));
        meta_class->SetSimpleField(15, "pOrigCrit", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, pOrigCrit_));
        meta_class->SetSimpleField(16, "vOrigCrit", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigCrit_));
        meta_class->SetSimpleField(17, "vOrigCriBon", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigCriBon_));
        meta_class->SetSimpleField(18, "vOrigDamageTakenRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigDamageTakenRateIn_));
        meta_class->SetSimpleField(19, "vOrigDamageDealtRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigDamageDealtRateIn_));
        meta_class->SetSimpleField(20, "vOrigNormalDamageTakenRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigNormalDamageTakenRateIn_));
        meta_class->SetSimpleField(21, "vOrigNormalDamageDealtRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigNormalDamageDealtRateIn_));
        meta_class->SetSimpleField(22, "vOrigSkillDamageTakenRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigSkillDamageTakenRateIn_));
        meta_class->SetSimpleField(23, "vOrigSkillDamageDealtRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigSkillDamageDealtRateIn_));
        meta_class->SetSimpleField(24, "vOrigHealthGainRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigHealthGainRateIn_));
        meta_class->SetSimpleField(25, "vOrigHealDealtRateIn", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigHealDealtRateIn_));
        meta_class->SetSimpleField(26, "vOrigSpeed", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigSpeed_));
        meta_class->SetSimpleField(27, "vOrigHaste", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigHaste_));
        meta_class->SetSimpleField(28, "vOrigDrain", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigDrain_));
        meta_class->SetSimpleField(29, "vOrigTenacity", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigTenacity_));
        meta_class->SetSimpleField(30, "vOrigBloDef", a8::reflect::ET_FLOAT, my_offsetof2(BattleBasicAttribute, vOrigBloDef_));
        meta_class->SetSimpleField(31, "battleGrowRule", a8::reflect::ET_STRING, my_offsetof2(BattleBasicAttribute, battleGrowRule_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleLevelAttribute::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleLevelAttribute", 4, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleLevelAttribute, id_));
        meta_class->SetSimpleField(1, "invoke", a8::reflect::ET_INT32, my_offsetof2(BattleLevelAttribute, invoke_));
        meta_class->SetSimpleField(2, "cost", a8::reflect::ET_STRING, my_offsetof2(BattleLevelAttribute, cost_));
        meta_class->SetSimpleField(3, "levelAttribute", a8::reflect::ET_STRING, my_offsetof2(BattleLevelAttribute, levelAttribute_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleRandAttribute::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleRandAttribute", 5, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleRandAttribute, id_));
        meta_class->SetSimpleField(1, "invoke", a8::reflect::ET_INT32, my_offsetof2(BattleRandAttribute, invoke_));
        meta_class->SetSimpleField(2, "qualityWeight", a8::reflect::ET_STRING, my_offsetof2(BattleRandAttribute, qualityWeight_));
        meta_class->SetSimpleField(3, "distribution", a8::reflect::ET_STRING, my_offsetof2(BattleRandAttribute, distribution_));
        meta_class->SetSimpleField(4, "attributeRange", a8::reflect::ET_STRING, my_offsetof2(BattleRandAttribute, attributeRange_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleWeaponAttribute::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleWeaponAttribute", 5, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleWeaponAttribute, id_));
        meta_class->SetSimpleField(1, "weapon", a8::reflect::ET_INT32, my_offsetof2(BattleWeaponAttribute, weapon_));
        meta_class->SetSimpleField(2, "level", a8::reflect::ET_INT32, my_offsetof2(BattleWeaponAttribute, level_));
        meta_class->SetSimpleField(3, "levelAttribute", a8::reflect::ET_STRING, my_offsetof2(BattleWeaponAttribute, levelAttribute_));
        meta_class->SetSimpleField(4, "cost", a8::reflect::ET_STRING, my_offsetof2(BattleWeaponAttribute, cost_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleUnit::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleUnit", 5, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleUnit, id_));
        meta_class->SetSimpleField(1, "weapon", a8::reflect::ET_INT32, my_offsetof2(BattleUnit, weapon_));
        meta_class->SetSimpleField(2, "level", a8::reflect::ET_INT32, my_offsetof2(BattleUnit, level_));
        meta_class->SetSimpleField(3, "levelAttribute", a8::reflect::ET_STRING, my_offsetof2(BattleUnit, levelAttribute_));
        meta_class->SetSimpleField(4, "cost", a8::reflect::ET_STRING, my_offsetof2(BattleUnit, cost_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Condition::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Condition", 6, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Condition, id_));
        meta_class->SetSimpleField(1, "param1", a8::reflect::ET_STRING, my_offsetof2(Condition, param1_));
        meta_class->SetSimpleField(2, "param2", a8::reflect::ET_STRING, my_offsetof2(Condition, param2_));
        meta_class->SetSimpleField(3, "param3", a8::reflect::ET_STRING, my_offsetof2(Condition, param3_));
        meta_class->SetSimpleField(4, "param4", a8::reflect::ET_STRING, my_offsetof2(Condition, param4_));
        meta_class->SetSimpleField(5, "param5", a8::reflect::ET_STRING, my_offsetof2(Condition, param5_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> Distribution::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("Distribution", 3, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(Distribution, id_));
        meta_class->SetSimpleField(1, "point", a8::reflect::ET_STRING, my_offsetof2(Distribution, point_));
        meta_class->SetSimpleField(2, "value", a8::reflect::ET_STRING, my_offsetof2(Distribution, value_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> LootConfig::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("LootConfig", 4, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(LootConfig, id_));
        meta_class->SetSimpleField(1, "method", a8::reflect::ET_INT32, my_offsetof2(LootConfig, method_));
        meta_class->SetSimpleField(2, "numPara", a8::reflect::ET_STRING, my_offsetof2(LootConfig, numPara_));
        meta_class->SetSimpleField(3, "content", a8::reflect::ET_STRING, my_offsetof2(LootConfig, content_));
      }
      return meta_class;
    }

    std::shared_ptr<a8::reflect::Class> BattleHeroGrow::GetClass() const
    {
      std::shared_ptr<a8::reflect::Class> meta_class = nullptr;
      if (!meta_class) {
        meta_class = std::make_shared<a8::reflect::Class>("BattleHeroGrow", 11, 0);
        meta_class->SetSimpleField(0, "id", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, id_));
        meta_class->SetSimpleField(1, "invoke", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, invoke_));
        meta_class->SetSimpleField(2, "heroId", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, heroId_));
        meta_class->SetSimpleField(3, "level", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, level_));
        meta_class->SetSimpleField(4, "levelExp", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, levelExp_));
        meta_class->SetSimpleField(5, "cumExp", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, cumExp_));
        meta_class->SetSimpleField(6, "lootExp", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, lootExp_));
        meta_class->SetSimpleField(7, "currentLevelAttribute", a8::reflect::ET_STRING, my_offsetof2(BattleHeroGrow, currentLevelAttribute_));
        meta_class->SetSimpleField(8, "currentEquipEffect", a8::reflect::ET_STRING, my_offsetof2(BattleHeroGrow, currentEquipEffect_));
        meta_class->SetSimpleField(9, "currentSkillEffect", a8::reflect::ET_INT32, my_offsetof2(BattleHeroGrow, currentSkillEffect_));
        meta_class->SetSimpleField(10, "currentSkillDesc", a8::reflect::ET_STRING, my_offsetof2(BattleHeroGrow, currentSkillDesc_));
      }
      return meta_class;
    }

}
