#include "precompile.h"

#include <f8/udplog.h>

#include "playermgr.h"
#include "player.h"
#include "room.h"

#include <f8/utils.h>
#include <f8/msgqueue.h>

#include "mt/Item.h"
#include "mt/Equip.h"

void PlayerMgr::Init()
{
}

void PlayerMgr::UnInit()
{
}

int PlayerMgr::OnlineNum()
{
    return socket_hash_.size();
}

Player* PlayerMgr::GetPlayerBySocket(int socket)
{
    auto itr = socket_hash_.find(socket);
    return itr != socket_hash_.end() ? itr->second : nullptr;
}

void PlayerMgr::OnGateDisconnect(int gg_socket)
{

    std::vector<int> socket_list;
    for (auto& pair : socket_hash_) {
        unsigned short parent_socket_handle = (pair.first >> 16) & 0xFFFF;
        if (parent_socket_handle == gg_socket) {
            socket_list.push_back(pair.first);
        }
    }
    for (int socket_handle : socket_list) {
        OnClientDisconnect(socket_handle);
    }
}

void PlayerMgr::OnClientDisconnect(int socket_handle)
{
    Player* hum = GetPlayerBySocket(socket_handle);
    if (hum) {
        RemovePlayerBySocket(socket_handle);
        hum->room->OnPlayerOffline(hum);
#if MYDEBUG
        a8::XPrintf("remove socket1 account_id%s %d\n", {hum->account_id, socket_handle});
#endif
    }
}

void PlayerMgr::RemovePlayerBySocket(int socket_handle)
{
    auto itr = socket_hash_.find(socket_handle);
    if (itr != socket_hash_.end()) {
#if MYDEBUG
        a8::XPrintf("RemovePlayerBySocket account_id%s room_uuid:%s %d\n",
                    {itr->second->account_id,
                     itr->second->room->GetRoomUuid(),
                     socket_handle});
#endif
        itr->second->SetSocketHandle(0);
        socket_hash_.erase(itr);
    }
}

void PlayerMgr::IncAccountNum(const std::string& account_id)
{
    auto itr = account_num_hash_.find(account_id);
    if (itr != account_num_hash_.end()) {
        ++(itr->second);
        if (itr->second > 3) {
            f8::UdpLog::Instance()->Warning
                (
                 "IncAccountNum account_id:%s num:%d > 3",
                 {
                  account_id,
                  itr->second
                 }
                 );
        }
    } else {
        account_num_hash_[account_id] = 1;
    }
}

void PlayerMgr::DecAccountNum(const std::string& account_id)
{
    auto itr = account_num_hash_.find(account_id);
    if (itr != account_num_hash_.end()) {
        --(itr->second);
        if (itr->second < 0) {
            f8::UdpLog::Instance()->Warning
                (
                 "DecAccountNum account_id:%s num:%d < 0",
                 {
                  account_id,
                  itr->second
                 }
                 );
        }
        if (itr->second <= 0) {
            account_num_hash_.erase(itr);
        }
    } else {
        f8::UdpLog::Instance()->Warning
            (
             "DecAccountNum account_id:%s not exits",
             {
                 account_id
             }
             );
    }
}

void PlayerMgr::ReBindSocket(Player* hum)
{
    if (!hum->SocketIsValid()) {
        A8_ABORT();
    }
#ifdef MYDEBUG
    a8::XPrintf("ReBindSocket account_id:%s socket_handle:%d\n",
                {
                 hum->account_id,
                 hum->GetSocketHandle(),
                });
#endif
    socket_hash_[hum->GetSocketHandle()] = hum;
}
