#pragma once

#include <a8/singleton.h>
#include <a8/timer_attacher.h>

#include <f8/protoutils.h>
#include <f8/timer.h>

namespace cs
{
    class CMJoin;
    class CMReconnect;
    class CMPing;
}

class GridService;
class MapService;
class MapInstance;
class CustomBattle;
class CustomMember;
struct RoomInitInfo
{
    int room_idx = 0;
    RoomMode_e room_mode = kPvpMode;
    std::string room_uuid;
    RoomType_e room_type = RoomType_NewBrid;
    int init_map_id = 0;
    int pve_instance_id = 0;
    int pve_human_num = 0;

    const mt::MapMode* map_mode_meta = nullptr;
    const mt::Map* map_meta = nullptr;
    std::shared_ptr<GridService> grid_service;
    std::shared_ptr<MapService> map_service;
    std::shared_ptr<MapInstance> map_instance;
    std::shared_ptr<CustomBattle> custom_battle;
};

class Room;
class RoomMgr : public a8::Singleton<RoomMgr>
{
 public:
    enum { HID = HID_RoomMgr };

 private:
    RoomMgr() {};
    friend class a8::Singleton<RoomMgr>;

 public:
    void Init();
    void UnInit();
    void Update(int delta_time);

    void _CMJoin(f8::MsgHdr* hdr, const cs::CMJoin& msg);
    void _CMReconnect(f8::MsgHdr* hdr, const cs::CMReconnect& msg);
    void _CMPing(f8::MsgHdr* hdr, const cs::CMPing& msg);
    void ActiveRoom(const std::string& room_uuid);
    int RoomNum();
    int OverRoomNum();
    std::shared_ptr<Room> GetRoomByUuid(const std::string& uuid);
    void AddOverRoom(const std::string& room_uuid);
    bool HasTask();
    void OnJoinRoomOk(const cs::CMJoin& msg, Player* hum);
    std::shared_ptr<CustomMember> GetCustomMemberBySocket(int socket_handle);
    void ReBindCustomMemberSocket(std::shared_ptr<CustomMember> p);
    void UnBindCustomMemberSocket(int socket_handle);
    std::shared_ptr<Room> GetJoinableRoom(std::shared_ptr<CustomBattle> p);
    std::shared_ptr<Room> CreateRoom(RoomType_e room_type,
                                     int mode_id,
                                     int map_id,
                                     RoomMode_e room_mode,
                                     std::shared_ptr<CustomBattle> custom_battle);

 private:
    void InstallReportStateTimer();
    std::shared_ptr<Room> GetRoomByIdx(int room_idx);
    void ReportServerState(int instance_id, const std::string& host, int port);
    void FreeOverRoom(const std::string& room_uuid);
    bool IsLimitJoin();

    int AllocRoomIdx();
    void JoinErrorHandle(const cs::CMJoin& msg, int error_code, int socket_handle);
    void AdjustCMJoin(cs::CMJoin* msg);
    std::shared_ptr<CustomBattle> GetCustomRoom(const std::string& room_uuid);
    std::shared_ptr<CustomBattle> GetHisCustomRoom(const std::string& room_uuid);
    void _CMJoinCustomBattle(f8::MsgHdr* hdr, const cs::CMJoin& msg, int custom_room_type);
    void SendGetCustomBattleData(std::shared_ptr<cs::CMJoin> join_msg,
                                 std::function<void(int, const std::string, std::shared_ptr<CustomBattle>)> cb,
                                 int custom_room_type);
    void DispatchSpecRoom(f8::MsgHdr* hdr, const cs::CMJoin& msg);

 private:
    int current_room_idx_ = 0;

    std::map<std::string, std::shared_ptr<Room>> inactive_room_hash_;
    std::map<std::string, std::shared_ptr<Room>> room_hash_;
    std::map<int, std::shared_ptr<Room>> room_idx_hash_;
    std::map<std::string, std::shared_ptr<Room>> over_room_hash_;
    f8::Attacher reportstate_timer_attacher_;
    std::map<std::string, std::shared_ptr<CustomBattle>> custom_room_hash_;
    std::map<std::string, std::shared_ptr<CustomBattle>> his_custom_room_hash_;
    std::map<int, std::shared_ptr<CustomMember>> socket_hash_;
};
