#include "precompile.h"
#include "weakptr.h"
#include "creature.h"
#include "perf.h"

CreatureWeakPtrChunk::CreatureWeakPtrChunk()
{
    INIT_LIST_HEAD(&ptrs_);
    ++Perf::Instance()->c_wptr_chunk_num;
}

CreatureWeakPtrChunk::~CreatureWeakPtrChunk()
{
    Clear();
    --Perf::Instance()->c_wptr_chunk_num;
}

void CreatureWeakPtrChunk::Clear()
{
    struct CreatureWeakPtr *weakptr, *tmp;
    struct list_head ptr_list;

    list_replace_init(&ptrs_, &ptr_list);
    list_for_each_entry_safe(weakptr, tmp, &ptr_list, entry_) {
        weakptr->Reset();
    }
}

void CreatureWeakPtrChunk::Set(Creature* c)
{
    ptr_ = c;
}

CreatureWeakPtr::CreatureWeakPtr()
{
    INIT_LIST_HEAD(&entry_);
    ++Perf::Instance()->c_wptr_num;
}

CreatureWeakPtr::CreatureWeakPtr(const CreatureWeakPtr& x)
{
    INIT_LIST_HEAD(&entry_);
    if (x.ptr_) {
        Attach(x.ptr_);
    }
    ++Perf::Instance()->c_wptr_num;
}

CreatureWeakPtr::CreatureWeakPtr(CreatureWeakPtr&& x)
{
    INIT_LIST_HEAD(&entry_);
    if (x.ptr_) {
        Attach(x.ptr_);
        x.Detach();
    }
    ++Perf::Instance()->c_wptr_num;
}

CreatureWeakPtr& CreatureWeakPtr::operator=(const CreatureWeakPtr& x)
{
    if (ptr_) {
        Detach();
    }
    if (x.ptr_) {
        Attach(x.ptr_);
    }
    return *this;
}

CreatureWeakPtr&& CreatureWeakPtr::operator=(CreatureWeakPtr&& x)
{
    if (ptr_) {
        Detach();
    }
    if (x.ptr_) {
        Attach(x.ptr_);
        x.Detach();
    }
    #if 1
    A8_ABORT();
    #else
    return *this;
    #endif
}

CreatureWeakPtr::~CreatureWeakPtr()
{
    Reset();
    --Perf::Instance()->c_wptr_num;
}

void CreatureWeakPtr::Reset()
{
    if (ptr_) {
        Detach();
    }
}

Creature* CreatureWeakPtr::Get()
{
    return ptr_;
}

void CreatureWeakPtr::Attach(Creature* c)
{
    Reset();
    CreatureWeakPtrChunk* chunk = c->GetWeakPtrChunk();
    if (c != chunk->ptr_) {
        A8_ABORT();
    }
    ptr_ = c;
    list_add_tail(&entry_, &chunk->ptrs_);
}

void CreatureWeakPtr::Detach()
{
    if (ptr_) {
        ptr_ = nullptr;
        list_del_init(&entry_);
    }
}

EntityWeakPtrChunk::EntityWeakPtrChunk()
{
    INIT_LIST_HEAD(&ptrs_);
    ++Perf::Instance()->e_wptr_chunk_num;
}

EntityWeakPtrChunk::~EntityWeakPtrChunk()
{
    Clear();
    --Perf::Instance()->e_wptr_chunk_num;
}

void EntityWeakPtrChunk::Clear()
{
    struct EntityWeakPtr *weakptr, *tmp;
    struct list_head ptr_list;

    list_replace_init(&ptrs_, &ptr_list);
    list_for_each_entry_safe(weakptr, tmp, &ptr_list, entry_) {
        weakptr->Reset();
    }
}

void EntityWeakPtrChunk::Set(Entity* c)
{
    ptr_ = c;
}

EntityWeakPtr::EntityWeakPtr()
{
    INIT_LIST_HEAD(&entry_);
    ++Perf::Instance()->e_wptr_num;
}

EntityWeakPtr::EntityWeakPtr(const EntityWeakPtr& x)
{
    INIT_LIST_HEAD(&entry_);
    if (x.ptr_) {
        Attach(x.ptr_);
    }
    ++Perf::Instance()->e_wptr_num;
}

EntityWeakPtr::EntityWeakPtr(EntityWeakPtr&& x)
{
    INIT_LIST_HEAD(&entry_);
    if (x.ptr_) {
        Attach(x.ptr_);
        x.Detach();
    }
    ++Perf::Instance()->e_wptr_num;
}

EntityWeakPtr& EntityWeakPtr::operator=(const EntityWeakPtr& x)
{
    if (ptr_) {
        Detach();
    }
    if (x.ptr_) {
        Attach(x.ptr_);
    }
    return *this;
}

EntityWeakPtr&& EntityWeakPtr::operator=(EntityWeakPtr&& x)
{
    if (ptr_) {
        Detach();
    }
    if (x.ptr_) {
        Attach(x.ptr_);
        x.Detach();
    }
    A8_ABORT();
    //    return *this;
}

EntityWeakPtr::~EntityWeakPtr()
{
    Reset();
    --Perf::Instance()->e_wptr_num;
}

void EntityWeakPtr::Reset()
{
    if (ptr_) {
        Detach();
    }
}

Entity* EntityWeakPtr::Get()
{
    return ptr_;
}

void EntityWeakPtr::Attach(Entity* c)
{
    Reset();
    EntityWeakPtrChunk* chunk = c->GetEntityWeakPtrChunk();
    if (c != chunk->ptr_) {
        A8_ABORT();
    }
    ptr_ = c;
    list_add_tail(&entry_, &chunk->ptrs_);
}

void EntityWeakPtr::Detach()
{
    if (ptr_) {
        ptr_ = nullptr;
        list_del_init(&entry_);
    }
}
