#include "precompile.h"

#include <unistd.h>

#include <f8/udplog.h>
#include <f8/btmgr.h>
#include <f8/timer.h>

#include "app.h"
#include "jsondatamgr.h"
#include "handlermgr.h"
#include "player.h"
#include "playermgr.h"
#include "httpproxy.h"
#include "tracemgr.h"
#include "iomgr.h"
#include "GGListener.h"
#include "perf.h"

#include "mt/MetaMgr.h"

void App::Init()
{
#ifdef MYDEBUG
    TraceMgr::Instance()->Init("robotserver2006");
#endif
    HandlerMgr::Instance()->Init();
    Perf::Instance()->Init();
    f8::BtMgr::Instance()->Init("exported");
#ifdef MYDEBUG1
    f8::BtMgr::Instance()->SetLogging(true);
#endif
    mt::MetaMgr::Instance()->Init();
    JsonDataMgr::Instance()->Init();
    IoMgr::Instance()->Init();
    HttpProxy::Instance()->Init();
    PlayerMgr::Instance()->Init();
    GGListener::Instance()->Init();
}

void App::UnInit()
{
    GGListener::Instance()->UnInit();
    PlayerMgr::Instance()->UnInit();
    HttpProxy::Instance()->UnInit();
    JsonDataMgr::Instance()->UnInit();
    f8::BtMgr::Instance()->UnInit();
    IoMgr::Instance()->UnInit();
    Perf::Instance()->UnInit();
    HandlerMgr::Instance()->UnInit();
#ifdef MYDEBUG
    TraceMgr::Instance()->UnInit();
#endif
}

bool App::HasTask()
{
    return true;
}

const std::string App::GetPkgName()
{
    return a8::Format("robotserver2006", {});
}

void App::Update(int delta_time)
{
    IoMgr::Instance()->Update();
    PlayerMgr::Instance()->Update();
}

void App::DispatchSocketMsg(f8::MsgHdr* hdr)
{
    f8::NetMsgHandler* handler = f8::GetNetMsgHandler
        (&HandlerMgr::Instance()->gsmsghandler,
         hdr->msgid);
    if (handler) {
        switch (hdr->sockfrom) {
        case MSF_GameGate:
            {
                auto hum = PlayerMgr::Instance()->GetPlayerBySocketHandle(hdr->socket_handle);
                if (hum) {
                    ProcessNetMsg(handler, hum.get(), hdr);
                }
            }
            break;
        default:
            {
            }
            break;
        }
    }
}
