#include "precompile.h"

#include <a8/websocketclient.h>
#include <a8/mutable_xobject.h>

#include <f8/udplog.h>
#include <f8/utils.h>
#include <f8/msgqueue.h>
#include <f8/comgr.h>
#include <f8/coroutine.h>
#include <f8/timer.h>

#include "playermgr.h"
#include "player.h"
#include "httpproxy.h"
#include "iomgr.h"
#include "app.h"

void PlayerMgr::Init()
{
    int count = 0;
    f8::Timer::Instance()->SetInterval
        (
         200,
         [this, count] (int et, const a8::Args* args) mutable
         {
             if (et == a8::TIMER_EXEC_EVENT) {
                 int idx = account_id_hash_.size() + 1;
                 std::string account_id = a8::Format("2_2006_%d", {idx});
                 auto hum = std::make_shared<Player>();
                 hum->Init(idx, account_id);
                 account_id_hash_[hum->GetAccountId()] = hum;
                 socket_id_hash_[hum->GetSocketId()] = hum;
                 ++count;
                 if (count >= 3) {
                     f8::Timer::Instance()->DeleteCurrentTimer();
                 }
             }
         });
}

void PlayerMgr::UnInit()
{
}

void PlayerMgr::Update()
{
    for (auto& pair : account_id_hash_) {
        pair.second->Update();
    }
}

std::shared_ptr<Player> PlayerMgr::GetPlayerByAccountId(const std::string& account_id)
{
    auto itr = account_id_hash_.find(account_id);
    return itr != account_id_hash_.end() ? itr->second : nullptr;
}

std::shared_ptr<Player> PlayerMgr::GetPlayerBySocketHandle(int socket_handle)
{
    auto itr = socket_id_hash_.find(socket_handle);
    return itr != socket_id_hash_.end() ? itr->second : nullptr;
}
