const axios = require('axios').default;
const ClientNet = require('./clientnet');

function getTickCount() {
  return Math.floor((new Date()).getTime() / 1);
}

function httpGet(url, params) {
  return new Promise((resolve) => {
    const ret = {
      err: null,
      response: null,
    };
    axios({
      method: 'get',
      url: url,
      timeout: 1000 * 5,
      params: params,
      responseType: 'text'
    }).then((response) => {
      ret.response = response.data;
      resolve(ret);
    }).catch((error) => {
      ret.err = error;
      resolve(ret);
    });
  });
}

async function sleep(timeout) {
  return new Promise(function (resolve, reject) {
    setTimeout(function () {
      resolve();
    }, timeout);
  });
}

class TestCase {

  constructor(url) {
    this.relationUrl = 'ws://192.168.100.45:7602/websocket?server_id=6';
    //this.relationUrl = 'ws://8.133.161.108/game1006/websocket/';
    this.loginData = null;
    this.gameConn = new ClientNet(
      this.relationUrl,
      'proto/cs_proto.proto',
      'proto/cs_msgid.proto'
    );
  }

  async init() {
    await this.step1();
    await this.step2();
    while (true) {
      await sleep(1000 * 3);
    }
  }

   async step1() {
    const startTick = getTickCount();
    const {err, response} = await httpGet(
      'https://login-test.kingsome.cn/webapp/index.php',
      {
        'c': 'Login',
        'a': 'auth',
        'channel': 6513,
        'login_type': 1001,
        'login_from': 1001,
        'gameid': 2006,
        'openid': 'e5vjg90hmboncfkqjDaiM8craNX23zPu'
      });
    this.loginData = response;
    const endTick = getTickCount();
    console.log('login auth@Login', endTick - startTick,  String(err), response);
   }

  async step2() {
    await this.gameConn.init();
    await this.gameConn.connect();

    this.gameConn.accountId = this.loginData['account_id'];
    this.gameConn.sessionId = this.loginData['session_id'];
    this.gameConn.sendMsg('CMJoin', {
      account_id: this.loginData['account_id'],
      session_id: this.loginData['session_id'],
    });
  }

}

module.exports = TestCase;
